/*********************************************************************************

Copyright(c) 2015 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
 * @file:    adi_cpt_v1.h
 * @brief:   Capture Timer service primary header file
 * @version: $Revision: 31408 $
 * @date:    $Date: 2015-06-11 14:00:16 -0400 (Thu, 11 Jun 2015) $
 *
 * @details
 *            This is the primary header file for the Capture Timer service
 *            which contains the API declarations, data and constant definitions 
 *            used in the APIs
 */

/** @addtogroup CPT_Driver Capture Timer Service
 *  @{
 */

#ifndef _ADI_CPT_V1_H_
#define _ADI_CPT_V1_H_

#include <stdint.h>
#include <stdbool.h>
#include <sys/platform.h>
#include <services/int/adi_int.h>

/*! CPTMR device handle */
typedef void* ADI_CPT_HANDLE;

/*! Memory required in bytes for the operation of each timer instance. */
#define ADI_CPT_MEMORY          	8u

/*! Enumeration of possible return codes from capture timer service.
 *
 *  These return codes may have different meaning with respect to different
 *  API. Please refer the description of each API for the description of the
 *  return codes with respect to that particular API.
 *
 */
typedef enum _ADI_CPT_RESULT_
{
    ADI_CPT_SUCCESS  = 0,           /*!< Success. */
    ADI_CPT_FAILURE,                /*!< Generic Failure. */
    ADI_CPT_INVALID_HANDLE,         /*!< If the given timer handle is invalid. */
    ADI_CPT_INVALID_MODE,           /*!< If the given timer mode is invalid. */
    ADI_CPT_IN_USE,                 /*!< If trying to open a timer which is already in use. */
    ADI_CPT_INVALID_POINTER,        /*!< If any of the given pointer parameters are invalid. */
    ADI_CPT_INVALID_TIMERID,        /*!< If the TIMER ID is out of range. */
    ADI_CPT_ALREADY_CLOSED,	    /*!< If trying to close the timer which is already closed. */
    ADI_CPT_INSUFFICIENT_MEMORY,    /*!< If the given memory is insufficient to operate the timer instance. */
    ADI_CPT_INIT_FAILED,            /*!< If the timer service initialization failed. */
    ADI_CPT_INT_FAILED,             /*!< If failed to register/unregister interrupt handler with inerrupt dispatcher. */
    ADI_CPT_INVALID_OPERATION,      /*!< If trying to configure the timer when it is running. */
    ADI_CPT_INVALID_PARAMETER       /*!< If one or more parameters are invalid. */
} ADI_CPT_RESULT;

/*! Enumeration of capture timer masks.  The masks are used to enable/disable
 * timers as a group.
 */
typedef enum _ADI_CPT_MASK_
{
    ADI_CPT_TMR0     =  0x0001u,           
    ADI_CPT_TMR1     =  0x0002u,           
    ADI_CPT_TMR2     =  0x0004u,           
    ADI_CPT_TMR3     =  0x0008u,           
    ADI_CPT_TMR4     =  0x0010u,           
    ADI_CPT_TMR5     =  0x0020u,           
    ADI_CPT_TMR6     =  0x0040u,           
    ADI_CPT_TMR7     =  0x0080u,           
    ADI_CPT_TMR8     =  0x0100u,           
    ADI_CPT_TMR9     =  0x0200u,           
    ADI_CPT_TMR10    =  0x0400u,           
    ADI_CPT_TMR11    =  0x0800u,           
    ADI_CPT_TMR12    =  0x1000u,           
    ADI_CPT_TMR13    =  0x2000u,           
    ADI_CPT_TMR14    =  0x4000u,           
    ADI_CPT_TMR15    =  0x8000u           
} ADI_CPT_MASK;

/*! Enumeration of different callback events generated by capture timer service */
typedef enum _ADI_CPT_EVENT_
{
    ADI_CPT_EVENT_TRIGGER_EDGE,      /*!< Leading edge of trigger pulse detected. */
    ADI_CPT_EVENT_COUNT_OVERFLOW     /*!< The count is exceeded resulting in a counter overflow. */
} ADI_CPT_EVENT;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * Public API's
 */

/* 
 * capture timer specific API's 
 */
  
/*
 * Opens a capture timer device.
 */
ADI_CPT_RESULT  adi_cpt_Open (
    uint32_t                  nTimerNum,
    void *              const pMemory,
    uint32_t                  nMemSize,
    ADI_CPT_HANDLE     *const phTimer
);

/*
 * Closes the given capture timer.
 */
ADI_CPT_RESULT  adi_cpt_Close (
    ADI_CPT_HANDLE    const hTimer
);

/*
 * Registers a user callback function with the driver.
 */
ADI_CPT_RESULT adi_cpt_RegisterCallback(
    ADI_CPT_HANDLE          const hTimer,
    ADI_CALLBACK            const pfCallback,
    void                   *const pCBParam
);

/*
 * Configure and enable the capture timer input pin.
 */
ADI_CPT_RESULT  adi_cpt_InputConfig(
    ADI_CPT_HANDLE          const hTimer,
    bool                          bAuxSel,
    bool                          bPolInvert,
    bool                          bEnable
);

/*
 * Configure and enable the capture timer trigger pin.
 */
ADI_CPT_RESULT  adi_cpt_TriggerConfig(
    ADI_CPT_HANDLE          const hTimer,
    bool                          bAuxSel,
    bool                          bPolInvert,    
    bool                          bEnable
);

/*
 * Gets the capture timer count.
 */
ADI_CPT_RESULT  adi_cpt_GetCount(
    ADI_CPT_HANDLE          const hTimer,
    uint32_t                     *pCount
);

/*
 * Gets the total ON time count of the input signal.
 */
ADI_CPT_RESULT  adi_cpt_GetOnTimeCount(
    ADI_CPT_HANDLE          const hTimer,
    uint32_t                     *pCount
);

/* 
 * capture timer common API's 
 */

/*
 * Start the capture timers.
 */
ADI_CPT_RESULT  adi_cpt_Start(
    uint16_t             nGroupMask
);

/*
 * Stop the capture timers.
 */
ADI_CPT_RESULT  adi_cpt_Stop(
    uint16_t             nGroupMask
);

/*
 * Enable/Disable the data interrupt.
 */
ADI_CPT_RESULT  adi_cpt_EnableDataInt(
    uint16_t             nGroupMask,
    bool                 bEnable
);

/*
 * Enable/Disable the status interrupt.
 */
ADI_CPT_RESULT  adi_cpt_EnableStatusInt(
    uint16_t             nGroupMask,
    bool                 bEnable
);

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* _ADI_CPT_V1_H_ */
/**@}*/
