/*!    @file adi_osal_arch_internal.h */
#ifndef __ADI_OSAL_ARCH_INTERNAL__
#define __ADI_OSAL_ARCH_INTERNAL__

#include "adi_osal.h" 
#include "adi_osal_arch.h"

/* Suppression for the ADSP-CM40x device.h file */
ADI_OSAL_MISRA_SUPPRESS_ALL
#include <device.h>
ADI_OSAL_MISRA_RESTORE_ALL

#include "osal_misra.h"

/* disable misra diagnostics as necessary 
 * 
 * Error[Pm123]: there shall be no definition of objects or functions in a 
 *                  header file (MISRA C 2004 rule 8.5)
 *
 * Error[Pm128]: illegal implicit conversion from underlying MISRA type  
 *                (MISRA C 2004 rule 10.1)
 * 
 * Error[Pm140]: a cast should not be performed between a pointer type and an 
 *                integral type (MISRA C 2004 rule 11.3)
 */
/*! @cond */
#if defined ( __ICCARM__ )
_Pragma ("diag_suppress= Pm001,Pm002,Pm123,Pm128,Pm140")
#endif
/*! @endcond */

/*! @addtogroup ADI_OSAL_Port_Specific ADI OSAL Port Specific
 *  @{
 */

/* external data (shared across OSAL environments) */

/*!
    @internal
    @var _adi_osal_gHandlerTable
    The OSAL dispatch table. It is an array of function pointers and callback arguments, 
    of the type corresponding to OSAL's plain C interrupt handlers (i.e. the high-level
    handlers).
    @endinternal
 */
extern ADI_OSAL_HANDLER_ENTRY _adi_osal_gHandlerTable[OSAL_HANDLER_TABLE_SIZE];

/*!
    @internal
    @var _adi_osal_gHandlerTableSize
    The size of the OSAL dispatch table. The size needs to be large enough for 
    any index that we can get back from adi_nvic_RegisterHandler(). 
    @endinternal
 */
extern uint32_t _adi_osal_gHandlerTableSize;

/* external code (shared across OSAL environments) */
extern ADI_OSAL_STATUS _adi_osal_HeapInstall(uint32_t *pHeapMemory, uint32_t nHeapMemorySize);
extern ADI_OSAL_STATUS _adi_osal_MemAlloc(void** ppData, uint32_t nSize);
extern void            _adi_osal_MemFree(void* pData);
extern bool_t          _adi_osal_IsCurrentLevelISR( void );
extern bool_t          _adi_osal_IsMemoryAligned(const void *pMemory);


extern void _adi_osal_stdWrapper(void);




/*!
  ****************************************************************************

   @internal
   @fn _adi_osal_IsCurrentLevelISR(void)
   @brief Describes whether the API is called at interrupt level or not

   @return true  - if the current execution level is greater than 0
   @return false - if the level is 0

   @endinternal
*****************************************************************************/
#pragma inline
bool_t _adi_osal_IsCurrentLevelISR( void )
{
    return (0ul != __get_IPSR()) ;
}

/*!
  ****************************************************************************
   @internal
   @fn _adi_osal_IsMemoryAligned(const void *pMemory)
   @brief This function indicates whether a pointer is aligned and can be used
          to store variables in the particular architecture

   @param[in] pMemory - Pointer to the allocated memory

   @return true    - if the memory was aligned
   @return false   - if the memory was not aligned

   @endinternal

*****************************************************************************/

#pragma inline
bool_t _adi_osal_IsMemoryAligned(const void *pMemory)
{
    return( 0u == ((uint32_t)(pMemory) & 0x3u));
}

/* enable misra diagnostics as necessary */
/*! @cond */
#if defined ( __ICCARM__ )
_Pragma ("diag_default= Pm001,Pm002,Pm123,Pm128,Pm140")
#endif
/*! @endcond */
/*@}*/

#endif /* __ADI_OSAL_ARCH_INTERNAL__ */
