/* $Revision: 30641 $
 * $Date: 2015-04-14 10:20:03 -0400 (Tue, 14 Apr 2015) $
******************************************************************************
Copyright (c), 2010-2013  - Analog Devices Inc. All Rights Reserved.
This software is PROPRIETARY & CONFIDENTIAL to Analog Devices, Inc.
and its licensors.
******************************************************************************

Title:      OSAL functions specific to Cortex-M4 & 
            OSAL environment FreeRTOS

Description:
              Operating System Abstraction Layer for the environment where
              there is FreeRTOS OS(Operating System) running.

*****************************************************************************/

/*!    @file adi_osal_freertos_arch_c.c */

/** @addtogroup ADI_OSAL_Port_Specific ADI OSAL Port Specific
 *  @{
 *
 * Operating System Abstraction Layer functions for the FreeRTOS 
 * environment on the cortex-m3
 */

#include "osal_misra.h"
   
#include <stdlib.h>     /* for NULL definition */
#include <limits.h>
#include "adi_osal.h"
#include "osal_freertos.h"
#include "adi_osal_arch.h"
#include "adi_osal_arch_internal.h"
#include "osal_common.h"

 /* disable misra diagnostics as necessary 
 * 
 * Error[Pm114]: an 'if (expression)' construct shall be followed by a compound 
 *              statement. The 'else' keyword shall be followed by either a 
 *              compound statement, or another 'if'  statement 
 *              (MISRA C 2004 rule 14.9)
 *
 * Error[Pm127]: a 'U' suffix shall be applied to all constants of 'unsigned' type 
 *                (MISRA C 2004 rule 10.6)
 *
 * Error[Pm128]: illegal implicit conversion from underlying MISRA type  
 *                (MISRA C 2004 rule 10.1)
 * 
 * Error[Pm140]: a cast should not be performed between a pointer type and an 
 *                integral type (MISRA C 2004 rule 11.3)
 */
/*! @cond */
#if defined ( __ICCARM__ )
_Pragma ("diag_suppress= Pm001,Pm002,Pm114,Pm127,Pm128,Pm140")
#endif
/*! @endcond */

/*!
  ****************************************************************************
    @fn _adi_osal_stdWrapper(void)
      
    @brief Standard wrapper for OSAL Interrupt Handlers in FreeRTOS. In FreeRTOS
    the context switching from an ISR (if needed) occurs by setting the pendSV bit 
    of the NVIC-ICSR register. Since the PendSV exception is assigned the lowest 
    priority the NVIC serves this interrupt when no other higher priority interrupt
    is pending to be served. This implies NVIC will activate PendSV only after
    the last of all nested interrupts completes. 
      
    Thus OSAL need not maintain the interrupt nesting counter to invoke 
    context switching since the NVIC will take care of nesting of all higher 
    priority interrupts and finally in the end the Pend SV interrupt will be
    served to perform context switching of task from an ISR  
      
    In FreeRTOS stdWrapper and plainWrapper implementations are the same but for
    future expansions keep them separate
    stdWrapper - wrapper function for callback handlers which inturn may call 
                 FreeRTOS FromISR APIs and which may cause a context swtich from
                 an ISR
                   
    plainWrapper - wrapper function for callback handlers for interrupts which 
                   don't support context switching from ISR. At present
                   it is used for exceptions and the non-maskable interrupt.

 *****************************************************************************/
void _adi_osal_stdWrapper (void)
{
    /* Get the interrupt number */
    uint32_t nIntNum = __get_IPSR();
       
     /* Call the higher level callback */
    _adi_osal_gHandlerTable[nIntNum].pfOSALHandler(ADI_NVIC_IRQ_SID(nIntNum),  _adi_osal_gHandlerTable[nIntNum].pOSALArg);

    return;
}

/* enable misra diagnostics as necessary */
/*! @cond */
#if defined ( __ICCARM__ )
_Pragma ("diag_default= Pm001,Pm002,Pm114,Pm127,Pm128,Pm140")
#endif
/*! @endcond */
/*@}*/