/* $Revision: 31505 $
 * $Date: 2015-06-19 06:21:02 -0400 (Fri, 19 Jun 2015) $
******************************************************************************
Copyright (c), 2008-2011 - Analog Devices Inc. All Rights Reserved.
This software is PROPRIETARY & CONFIDENTIAL to Analog Devices, Inc.
and its licensors.
******************************************************************************/
/*!
    @file adi_osal_freertos_mutex.c

  Operating System Abstraction Layer - OSAL for FreeRTOS - Mutex functions

*/
/** @addtogroup ADI_OSAL_Mutex ADI OSAL Mutex
 *  @{
 * This module contains the Mutex APIs for the FreeRTOS implementation of OSAL

 * @note: priority inheritance is always enabled in FreeRTOS
 */

/*=============  I N C L U D E S   =============*/

#include "osal_misra.h"
#include "adi_osal.h"
#include "osal_freertos.h"
#include "osal_common.h"

#include <limits.h>
#include <stdlib.h>

/* disable misra diagnostics as necessary
 * Error[Pm073]:  a function should have a single point of exit
 *               (MISRA C 2004 rule 14.7)
 *
 * Error[Pm114]: an 'if (expression)' construct shall be followed by a compound
 *              statement. The 'else' keyword shall be followed by either a
 *              compound statement, or another 'if'  statement
 *              (MISRA C 2004 rule 14.9)
 *
 * Error[Pm140]: a cast should not be performed between a pointer type and an
 *                integral type (MISRA C 2004 rule 11.3)
 *
 * Error[Pm141]: a cast should not be performed between a pointer to object type
 *                  and a different pointer to object type(MISRA C 2004 rule 11.4)
 *
 * Error[Pm143]: a function should have a single point of exit at the end of
 *                the function (MISRA C 2004 rule 14.7)
 */
/*! @cond */
#if defined ( __ICCARM__ )
_Pragma ("diag_suppress= Pm001,Pm002,Pm073,Pm114,Pm140,Pm141,Pm143")
#endif
/*! @endcond */



/*=============  D A T A  =============*/




/*=============  C O D E  =============*/



/*!
  ****************************************************************************
    @brief Creates a mutex.

    @param[out] phMutex  - Pointer to a location to write the returned mutex
                           handle

    @return ADI_OSAL_SUCCESS      - If mutex is created successfully
    @return ADI_OSAL_FAILED       - If failed to create mutex
    @return ADI_OSAL_MEM_ALLOC_FAILED - If memory could not be allocated
    @return ADI_OSAL_CALLER_ERROR - If the function is invoked from an invalid
                                    location
    @return ADI_OSAL_BAD_MEMORY       - If the allocated memory is not word aligned.
    @return ADI_OSAL_MEM_TOO_SMALL    - If there isn't sufficient memory to create
                                           the mutex
     "phMutex" points to NULL if mutex creation is failed.

 *****************************************************************************/

ADI_OSAL_STATUS adi_osal_MutexCreate(ADI_OSAL_MUTEX_HANDLE *phMutex)
{
    ADI_OSAL_STATUS eRetStatus;
    ADI_OSAL_MUTEX_INFO_PTR pMutexObj;

    if (NULL == phMutex)
    {
        return(ADI_OSAL_FAILED);
    }

    if (CALLED_FROM_AN_ISR)
    {
        *phMutex = ADI_OSAL_INVALID_MUTEX;
        return (ADI_OSAL_CALLER_ERROR);
    }

    /*a new mutex is created, we need to allocate some memory for it */
    eRetStatus = _adi_osal_MemAlloc((void**) &pMutexObj, sizeof(ADI_OSAL_MUTEX_INFO));

    if (ADI_OSAL_SUCCESS != eRetStatus)
    {
        *phMutex = ADI_OSAL_INVALID_MUTEX;
        return (eRetStatus);
    }

    eRetStatus = adi_osal_MutexCreateStatic(pMutexObj, sizeof(ADI_OSAL_MUTEX_INFO), phMutex);

    if (eRetStatus != ADI_OSAL_SUCCESS)
    {
        _adi_osal_MemFree(pMutexObj);
    }

    return (eRetStatus);
}

/*!
  ****************************************************************************
    @brief Creates a mutex using user allocated memory for the mutex

    THIS FUNCTION IS THE SAME AS adi_osal_MutexCreate except that memory is
    passed to it instead of relying on dynamic memory.

    @param[out] phMutex      - Pointer to a location to write the returned
                               mutex handle
    @param[in] nMutexObjSize - Size of the memory passed for the creation of
                               the mutex
    @param[in] pMutexObject  - Area of memory provided to us for the mutex

    @return ADI_OSAL_SUCCESS            - If mutex is created successfully
    @return ADI_OSAL_FAILED             - If failed to create mutex
    @return ADI_OSAL_MEM_TOO_SMALL    - If there isn't sufficient memory to create
                                           the mutex
    @return ADI_OSAL_CALLER_ERROR       - If the function is invoked from an invalid
                                        location
    @return ADI_OSAL_BAD_MEMORY       - If the allocated memory is not word aligned.

    @see adi_osal_MutexCreate
    @see adi_osal_MutexDestroyStatic
 *****************************************************************************/

ADI_OSAL_STATUS adi_osal_MutexCreateStatic(void* const pMutexObject, uint32_t nMutexObjSize, ADI_OSAL_MUTEX_HANDLE *phMutex)
{
    SemaphoreHandle_t       hMtxNative;

    if (NULL == phMutex)
    {
        return(ADI_OSAL_FAILED);
    }

    if (CALLED_FROM_AN_ISR)
    {
        *phMutex = ADI_OSAL_INVALID_MUTEX;
        return (ADI_OSAL_CALLER_ERROR);
    }

#ifdef OSAL_DEBUG
    if ( nMutexObjSize < sizeof(ADI_OSAL_MUTEX_INFO) )
    {
        *phMutex = ADI_OSAL_INVALID_MUTEX;
        return (ADI_OSAL_MEM_TOO_SMALL);
    }

    if (!_adi_osal_IsMemoryAligned(pMutexObject))
    {
        *phMutex = ADI_OSAL_INVALID_MUTEX;
        return (ADI_OSAL_BAD_MEMORY);
    }

#endif /* OSAL_DEBUG */

    hMtxNative = xSemaphoreCreateMutex();
    if ( hMtxNative != NULL)
    {
        ADI_OSAL_MUTEX_INFO_PTR pMutex = (ADI_OSAL_MUTEX_INFO_PTR) pMutexObject;

        pMutex->hMutexID = hMtxNative;
        pMutex->hOwnerThread = NULL; /* the mutex is not owned by the thread */
                                     /* creating it but by the thread        */
                                     /* acquiring it (MutexPend)             */
        pMutex->nAcquisitionCount = 0u; /* initially a mutex is not acquired */

        *phMutex = (ADI_OSAL_MUTEX_HANDLE) pMutex;

        return (ADI_OSAL_SUCCESS);
    }
    else
    {
        *phMutex = ADI_OSAL_INVALID_MUTEX;
        return (ADI_OSAL_FAILED);
    }
}

/*!
  ****************************************************************************
  @brief Returns the size of a mutex object.

  This function can be used by the adi_osal_MutexCreateStatic function to
  determine what the object size should be for this particular RTOS
  implementation.

  Parameters:
      None

    @return size of a mutex object in bytes.



*****************************************************************************/

uint32_t adi_osal_MutexGetObjSize(void)
{
    return ( sizeof(ADI_OSAL_MUTEX_INFO) );
}

/*!
  ****************************************************************************
    @brief Deletes a mutex without freeing memory

    This API is designed to destroy a mutex that has been allocated with
    adi_osal_MutexCreateStatic().

    @param[in] hMutex      - Handle of the mutex to be deleted

    @return ADI_OSAL_SUCCESS      - If mutex is deleted successfully
    @return ADI_OSAL_BAD_HANDLE   - If the specified mutex handle is invalid
    @return ADI_OSAL_CALLER_ERROR - If function is invoked from an invalid
                                    location
    @note
      Only owner is authorised to release the acquired mutex. But it
      can "destroyed" by  other task.

    @see adi_osal_MutexCreateStatic
    @see adi_osal_MutexDestroy

*****************************************************************************/

ADI_OSAL_STATUS adi_osal_MutexDestroyStatic(ADI_OSAL_MUTEX_HANDLE const hMutex)
{
    ADI_OSAL_STATUS eRetStatus;
    ADI_OSAL_MUTEX_INFO_PTR  pMutexInfo  = (ADI_OSAL_MUTEX_INFO_PTR) hMutex;

    if(CALLED_FROM_AN_ISR)
    {
      return (ADI_OSAL_CALLER_ERROR);
    }

    if ((NULL == hMutex) || (ADI_OSAL_INVALID_MUTEX == hMutex))
    {
      return (ADI_OSAL_BAD_HANDLE);
    }

    vSemaphoreDelete(pMutexInfo->hMutexID);
    eRetStatus = ADI_OSAL_SUCCESS;

    return(eRetStatus);
}

/*!
  ****************************************************************************
    @brief This function is used to delete a mutex.

    @param[in] hMutex      - Handle of the mutex to be deleted

    @return ADI_OSAL_SUCCESS      - If mutex is deleted successfully
    @return ADI_OSAL_BAD_HANDLE   - If the specified mutex handle is invalid
    @return ADI_OSAL_CALLER_ERROR - If function is invoked from an invalid
                                    location
      @note
      Only owner is authorised to release the acquired mutex. But it
      can "destroyed" by  other task.

*****************************************************************************/
ADI_OSAL_STATUS adi_osal_MutexDestroy(ADI_OSAL_MUTEX_HANDLE const hMutex)
{
    ADI_OSAL_STATUS eRetStatus;
    ADI_OSAL_MUTEX_INFO_PTR  pMutexInfo  = (ADI_OSAL_MUTEX_INFO_PTR) hMutex;

    eRetStatus = adi_osal_MutexDestroyStatic(hMutex);
    if (eRetStatus == ADI_OSAL_SUCCESS )
    {
        _adi_osal_MemFree(pMutexInfo);
    }

    return (eRetStatus);
}


/*!
  ****************************************************************************
  @brief Acquires a mutex with a timeout

  This function is used to lock a mutex (acquire a resource)

    @param[in] hMutex            - Handle of the mutex which need to be acquired
    @param[in] nTimeoutInTicks   - Specify the number of system ticks for
                                   acquiring the mutex

      Valid timeouts are:

        ADI_OSAL_TIMEOUT_NONE       -  No wait. Results in an immediate return
                                       from this service  regardless of whether
                                       or not it was successful

        ADI_OSAL_TIMEOUT_FOREVER    -  Wait option for calling task to suspend
                                       indefinitely until a specified  mutex is
                                       obtained

        1 ....0xFFFFFFFE            -  Selecting a numeric value specifies the
                                       maximum time limit (in system ticks) for
                                       obtaining specified mutex

    @return ADI_OSAL_SUCCESS      - If the specified mutex is locked
                                    successfully
    @return ADI_OSAL_TIMEOUT      - If the specified time limit expired.
    @return ADI_OSAL_BAD_HANDLE   - If the specified mutex ID is invalid
    @return ADI_OSAL_CALLER_ERROR - If the function is invoked from an invalid
                                    location
    @return ADI_OSAL_BAD_TIME     - If the timeout value is invalid
    @return ADI_OSAL_FAILED       - If pend operation is failed due to timeout


*****************************************************************************/

ADI_OSAL_STATUS adi_osal_MutexPend(ADI_OSAL_MUTEX_HANDLE const hMutex, ADI_OSAL_TICKS nTimeoutInTicks)
{
    BaseType_t   nRetValue;
    ADI_OSAL_STATUS  eRetStatus;
    TickType_t nTimeout = 0u;
    ADI_OSAL_MUTEX_INFO_PTR  pMutexInfo = (ADI_OSAL_MUTEX_INFO_PTR) hMutex;

#ifdef OSAL_DEBUG
    if((nTimeoutInTicks > ADI_OSAL_MAX_TIMEOUT) &&
       (nTimeoutInTicks != ADI_OSAL_TIMEOUT_FOREVER))
    {
        return(ADI_OSAL_BAD_TIME);
    }
#endif /* OSAL_DEBUG */

    if (CALLED_BEFORE_OS_RUNNING)
    {
        return (ADI_OSAL_CALLER_ERROR);
    }

     if (CALLED_FROM_AN_ISR)
     {
       return (ADI_OSAL_CALLER_ERROR);
     }

    if ((NULL == hMutex) || (ADI_OSAL_INVALID_MUTEX == hMutex))
    {
        return (ADI_OSAL_BAD_HANDLE);
    }

    /* Current thread handle is necessary if all following conditionals paths */
    /* The mutex owner is a FreeRTOS thread pointer because all threads in the
     * application are FreeRTOS threads but not all threads are OSAL threads
     */
    switch (nTimeoutInTicks)
    {
        case ADI_OSAL_TIMEOUT_NONE:
            nTimeout = FREERTOS_NONBLOCKING_CALL;
            break;
        case ADI_OSAL_TIMEOUT_FOREVER:
            nTimeout = (TickType_t)portMAX_DELAY ;
            break;
        default:
            nTimeout = (TickType_t)nTimeoutInTicks ;
            break;
     }  /* end of switch */

    nRetValue = xSemaphoreTake( pMutexInfo->hMutexID, nTimeout );

    switch (nRetValue)
    {
        case  pdTRUE :
            eRetStatus = ADI_OSAL_SUCCESS;
            break;

        case errQUEUE_EMPTY:
            eRetStatus = ADI_OSAL_TIMEOUT;
            break;
            
        default:
            eRetStatus = ADI_OSAL_FAILED;
            break;
    }

    return (eRetStatus);
}



/*!
  ****************************************************************************

    @brief Unlocks a mutex.

    @param[in] hMutex      - Handle of the mutex which needs to be unlocked

    @return ADI_OSAL_SUCCESS          - If mutex is un locked successfully
    @return ADI_OSAL_FAILED           - If failed unlock mutex
    @return ADI_OSAL_BAD_HANDLE       - If the specified mutex ID is invalid
    @return ADI_OSAL_CALLER_ERROR     - If the call is made from an invalid location
                                        (i.e an ISR)

    @note
         Mutex can be successfully released by its owner : Only the task which
         acquired it can release it. Any attempt to release it by non-owner will
         result in error.

*****************************************************************************/

ADI_OSAL_STATUS adi_osal_MutexPost(ADI_OSAL_MUTEX_HANDLE const hMutex)
{
    BaseType_t nRetValue;
    ADI_OSAL_STATUS eRetStatus;
    ADI_OSAL_MUTEX_INFO_PTR  pMutexInfo  = (ADI_OSAL_MUTEX_INFO_PTR) hMutex;

    /* check whether "pMutexInfo" is a valid "mutex" and that it is called from
       the thread which acquired it.
       Irrespective of priority, A mutex can be "released" only by the thread
       which owns it.
    */
    if ((NULL == hMutex) || (ADI_OSAL_INVALID_MUTEX == hMutex))
    {
        return (ADI_OSAL_BAD_HANDLE);
    }

    if (CALLED_FROM_AN_ISR)
    {
        return (ADI_OSAL_CALLER_ERROR);
    }

    nRetValue = xSemaphoreGive(pMutexInfo->hMutexID);

    if(pdTRUE == nRetValue)
    {
        eRetStatus = ADI_OSAL_SUCCESS;
    }
    else
    {
        eRetStatus = ADI_OSAL_FAILED;
    }

    return(eRetStatus);
}

/* enable misra diagnostics as necessary */
/*! @cond */
#if defined ( __ICCARM__ )
_Pragma ("diag_default= Pm001,Pm002,Pm073,Pm114,Pm140,Pm141,Pm143")
#endif
/*! @endcond */
/*
**
** EOF:
**
*/
/*@}*/
