/* $Revision:  0$
 * $Date: 2015-04-14 10:20:03 -0400 (Tue, 14 Apr 2015) $
******************************************************************************
Copyright (c), 2010-2013 - Analog Devices Inc. All Rights Reserved.
This software is PROPRIETARY & CONFIDENTIAL to Analog Devices, Inc.
and its licensors.
******************************************************************************/
/*!
    @file adi_osal_freertos_thread.c

    Operating System Abstraction Layer - OSAL for FreeRTOS - Thread
    functions


*/
/** @addtogroup ADI_OSAL_Threads ADI OSAL Threads
 *  @{
 *
 * This module contains the Thread APIs for the FreeRTOS implementation of
 * OSAL.
 */
/*=============  I N C L U D E S   =============*/

#include <string.h>                                                             /* for strncpy */
#include <stddef.h>

#include "osal_misra.h"
#include "adi_osal.h"
#include "osal_freertos.h"
#include "osal_common.h"

/* disable misra diagnostics as necessary
 * Error[Pm073]:  a function should have a single point of exit
 *               (MISRA C 2004 rule 14.7)
 *
 * Error[Pm088]: pointer arithmetic should not be used (MISRA C 2004 rule 17.4)
 *
 * Error[Pm127]: a 'U' suffix shall be applied to all constants of 'unsigned' type
 *                (MISRA C 2004 rule 10.6)
 *
 * Error[Pm128]: illegal implicit conversion from underlying MISRA type
 *                (MISRA C 2004 rule 10.1)
 *
 * Error[Pm138]: conversions shall not be performed between a pointer to a
 *                 function and any other type other than an integral type
 *                 (MISRA C 2004 rule 11.1)
 *
 * Error[Pm139]: conversions shall not be performed between a pointer to object
 *                 and any type other than an integral type
 *                (MISRA C 2004 rule 11.2)
 *
 * Error[Pm140]: a cast should not be performed between a pointer type and an
 *                integral type (MISRA C 2004 rule 11.3)
 *
 * Error[Pm141]: a cast should not be performed between a pointer to object type
 *                  and a different pointer to object type(MISRA C 2004 rule 11.4)
 *
 * Error[Pm143]: a function should have a single point of exit at the end of
 *                the function (MISRA C 2004 rule 14.7)
 */
/*! @cond */
#if defined ( __ICCARM__ )
_Pragma ("diag_suppress= Pm001,Pm002,Pm073,Pm088,Pm127,Pm128,Pm138,Pm139,Pm140,Pm141,Pm143")
#endif
/*! @endcond */
/*=============  D A T A  =============*/




/*=============  C O D E  =============*/



/*!
  ****************************************************************************
    @brief Creates a thread and puts it in the ready state.

    @param[in] pThreadAttr - Pointer to the (pre-initialized)
                             ADI_OSAL_THREAD_ATTR structure
    @param[out] phThread   - Pointer to a location to return the thread handle
                             if the thread is created successfully

    @return ADI_OSAL_SUCCESS              - if thread creation is successful
    @return ADI_OSAL_FAILED               - if thread creation fails
    @return ADI_OSAL_BAD_THREAD_FUNC      - if the specified function address is Null
    @return ADI_OSAL_BAD_STACK_SIZE       - Stack size is not word aligned
    @return ADI_OSAL_BAD_THREAD_NAME      - If the specified thread name exceeds
                                            ADI_OSAL_MAX_THREAD_NAME
    @return ADI_OSAL_CALLER_ERROR         - if function is invoked from an
                                            invalid location (i.e an ISR)
    @return ADI_OSAL_MEM_ALLOC_FAILED     - Memory allocation for the RTOS object
                                            failed
*****************************************************************************/

ADI_OSAL_STATUS adi_osal_ThreadCreate(ADI_OSAL_THREAD_HANDLE *phThread, const ADI_OSAL_THREAD_ATTR *pThreadAttr)
{
    BaseType_t     nRetValue;
    uint16_t nStkSize;
    ADI_OSAL_STATUS eRetStatus;
    ADI_OSAL_THREAD_INFO_PTR hThreadNode;
    UBaseType_t  nAssignedPrio;
    ADI_OSAL_STATUS eHeapResult;

#ifdef OSAL_DEBUG
    if (NULL == phThread)
    {
        return(ADI_OSAL_FAILED);
    }

    /* verify that the given structure is not NULL before starting to access the fields */
    if (NULL == pThreadAttr)
    {
        *phThread = ADI_OSAL_INVALID_THREAD;
        return(ADI_OSAL_FAILED);
    }

    if (NULL == pThreadAttr->pThreadFunc)
    {
        *phThread = ADI_OSAL_INVALID_THREAD;
        return(ADI_OSAL_BAD_THREAD_FUNC);
    }

    if (0u == pThreadAttr->nStackSize)
    {
        *phThread = ADI_OSAL_INVALID_THREAD;
        return (ADI_OSAL_BAD_STACK_SIZE);
    }

    if (!_adi_osal_IsMemoryAligned((void*) pThreadAttr->nStackSize))
    {
        *phThread = ADI_OSAL_INVALID_THREAD;
        return (ADI_OSAL_BAD_STACK_SIZE);
    }

    if (strlen(pThreadAttr->szThreadName) > ADI_OSAL_MAX_THREAD_NAME)
    {
        *phThread = ADI_OSAL_INVALID_THREAD;
        return ( ADI_OSAL_BAD_THREAD_NAME);
    }

#endif
    /* FreeRTOS API cannot capture called from ISR Error, hence move it outside
     * OSAL_DEBUG */
    if (CALLED_FROM_AN_ISR)
    {
        *phThread = ADI_OSAL_INVALID_THREAD;
        return(ADI_OSAL_CALLER_ERROR);
    }

    /* Convert the stack size from bytes to elements of size_t */
    nStkSize = (uint16_t)(pThreadAttr->nStackSize/sizeof(size_t));

    /*  ADI_OSAL_FREERTOS_BASE_PRIO is the idle task priority defined
     *  to tskIDLE_PRIORITY */
    nAssignedPrio= pThreadAttr->nPriority + ADI_OSAL_FREERTOS_BASE_PRIO;

    /* Create a object of type ADI_OSAL_THREAD_INFO which will serve
     * as the handle for the OSAL thread */

    eHeapResult = _adi_osal_MemAlloc((void**) &hThreadNode, sizeof(ADI_OSAL_THREAD_INFO));

    if (ADI_OSAL_SUCCESS != eHeapResult)
    {
        *phThread = ADI_OSAL_INVALID_THREAD;
        return (eHeapResult);
    }

    /* Creating a task  by calling native OS call.*/
    /* we set the signature first because if the creation succeeds and the
      new thread has higher priority we won't come back
    */
    hThreadNode->nThreadSignature = ADI_OSAL_THREAD_SIGNATURE;

    /* The scheduler needs to be locked so that the TaskCreation and the Task
       Tag assignement can be done atomically */
    vTaskSuspendAll();

    /* Creating a task  by calling native OS call */
    nRetValue = xTaskCreate ((TaskFunction_t) pThreadAttr->pThreadFunc,
                                 pThreadAttr->szThreadName,
                                 nStkSize,
                                 pThreadAttr->pTaskAttrParam,
                                 nAssignedPrio,
                                 &(hThreadNode->pNativeThread));

    switch (nRetValue)
    {
        case  pdPASS :

          /* vTaskSetApplicationTaskTag is used for storing the pointer to the
          * ADI_OSAL_THREAD_INFO structure so that it can be found when
          * given the FreeRTOS task handle
         */
          vTaskSetApplicationTaskTag((hThreadNode->pNativeThread), (TaskHookFunction_t) hThreadNode);

          /* The ADI_OSAL_THREAD_INFO structure is used as the thread handle */
             *phThread = (ADI_OSAL_THREAD_HANDLE) hThreadNode;
            eRetStatus = ADI_OSAL_SUCCESS;

            break;

#ifdef OSAL_DEBUG
        /* Not enough memory to create task */
        case  errCOULD_NOT_ALLOCATE_REQUIRED_MEMORY:
            eRetStatus = ADI_OSAL_MEM_ALLOC_FAILED;
            break;
#endif
        default:
            eRetStatus = ADI_OSAL_FAILED;
            break;
    } /* end of switch */

    xTaskResumeAll();
    if(eRetStatus != ADI_OSAL_SUCCESS)
    {
        *phThread = ADI_OSAL_INVALID_THREAD;
        if (NULL != hThreadNode)
        {
            hThreadNode->nThreadSignature=ADI_OSAL_INVALID_THREAD_SIGNATURE;
            _adi_osal_MemFree(hThreadNode);
        }
    }

    return(eRetStatus);
}


/*!
  ****************************************************************************
    @brief Deletes a thread (hence can no longer be scheduled)
           ADI Osal ThreadDestroy is used for deleting other threads
           It doesn't support self delete

    @param[in] hThread - Handle of the thread to be deleted

    @return ADI_OSAL_SUCCESS      -  If successfully removed the thread from
                                     the system
    @return ADI_OSAL_FAILED       -  If failed to delete the thread
    @return ADI_OSAL_BAD_HANDLE   -  If the specified thread handle is invalid
    @return ADI_OSAL_CALLER_ERROR -  If function is invoked from an invalid
                                     location (i.e an ISR)

*****************************************************************************/

ADI_OSAL_STATUS  adi_osal_ThreadDestroy(ADI_OSAL_THREAD_HANDLE const hThread)
{
    ADI_OSAL_STATUS eRetStatus = ADI_OSAL_FAILED;
    ADI_OSAL_THREAD_INFO_PTR hThreadNode = (ADI_OSAL_THREAD_INFO_PTR) hThread;
    TaskHandle_t hTaskHandle = hThreadNode->pNativeThread;
    TaskHandle_t hCurrNativeThread;

#ifdef OSAL_DEBUG
    /* check validity of the handle */
    if ((NULL == hThreadNode) || (ADI_OSAL_INVALID_THREAD == hThread))
    {
        return (ADI_OSAL_BAD_HANDLE);
    }

    if ( !_adi_osal_IsOSALThread(hThreadNode) )
    {
        return (ADI_OSAL_BAD_HANDLE);
    }
#endif

    /* FreeRTOS API cannot capture Bad Handle, hence move it outside
     * OSAL_DEBUG */
    if (ADI_OSAL_INVALID_THREAD == hThread)
    {
        return (ADI_OSAL_BAD_HANDLE);
    }

    /* FreeRTOS API cannot capture called from ISR Error, hence move it outside
     * OSAL_DEBUG */
    if (CALLED_FROM_AN_ISR)
    {
        return (ADI_OSAL_CALLER_ERROR);
    }


   eRetStatus = adi_osal_ThreadGetNativeHandle((void**) (&hCurrNativeThread));
    if (ADI_OSAL_SUCCESS != eRetStatus)
    {
        return (ADI_OSAL_FAILED);
    }

    vTaskDelete(hTaskHandle);

    hThreadNode->nThreadSignature=ADI_OSAL_INVALID_THREAD_SIGNATURE;
    _adi_osal_MemFree(hThreadNode);


    return(ADI_OSAL_SUCCESS);
}



/*!
  ****************************************************************************
    @brief Returns the native handle of the current thread.

    @param[in] phThread - pointer to a location to write the current thread
                          native handle upon successful.

    @return ADI_OSAL_SUCCESS  - if the OS was running and there is a thread
    @return ADI_OSAL_FAILED   - if location handle address is invalid or this API
                                is called before the OS has started where no threads
                                are running
    @note "phThread" will be set to "ADI_OSAL_INVALID_THREAD" if not successful.

    @see adi_osal_ThreadGetHandle
*****************************************************************************/

ADI_OSAL_STATUS  adi_osal_ThreadGetNativeHandle(void **phThread)
{
    TaskHandle_t hCurrentTaskHandle;

#ifdef OSAL_DEBUG
    if (NULL == phThread)
    {
        return(ADI_OSAL_FAILED);
    }
#endif

    if (CALLED_BEFORE_OS_RUNNING)
    {
        * (ADI_OSAL_THREAD_HANDLE*) phThread = ADI_OSAL_INVALID_THREAD;
        return(ADI_OSAL_FAILED);
    }

    /*Call FreeRTOS API to retrive the current task handle*/
    hCurrentTaskHandle = xTaskGetCurrentTaskHandle();
    *( TaskHandle_t * ) phThread = hCurrentTaskHandle ;

    return (ADI_OSAL_SUCCESS);
}

/*!
  ****************************************************************************
    @brief Returns the handle of the current thread if it is an OSAL thread.

    @param[in] phThread - pointer to a location to write the current OSAL thread
                          handle upon successful.

    @return ADI_OSAL_SUCCESS if the current thread was created with OSAL
    @return ADI_OSAL_FAILED  if the current thread was not created with OSAL

    @note "phThread" will be set to "ADI_OSAL_INVALID_THREAD" if not successful.

    @see adi_osal_ThreadGetNativeHandle
*****************************************************************************/

/* a typecast is necessary here because the thread handle type is incomplete
 * and is just an abstract pointer, the real structure (the type of
 * _adi_osal_oStartupVirtualThread) cannot be used directly because it is
 * hidden from the public interface. */

ADI_OSAL_STATUS  adi_osal_ThreadGetHandle(ADI_OSAL_THREAD_HANDLE *phThread)
{
    ADI_OSAL_THREAD_INFO_PTR hThreadNode;

    if (CALLED_BEFORE_OS_RUNNING)
    {
        *phThread  = (ADI_OSAL_THREAD_HANDLE) &_adi_osal_oStartupVirtualThread;
        return (ADI_OSAL_SUCCESS);
    }

    /* Check if the current thread is an OSAL thread by looking at the task tag
    * memory address where the signature should be.
    */
    hThreadNode = (ADI_OSAL_THREAD_INFO_PTR) xTaskGetApplicationTaskTag(NULL);

#ifdef OSAL_DEBUG

    if ( !_adi_osal_IsOSALThread(hThreadNode) )
    {
        return (ADI_OSAL_BAD_HANDLE);
    }

#endif

    /* The current thread was an OSAL thread so we return the OSAL handle */
    *phThread  =(ADI_OSAL_THREAD_HANDLE)(hThreadNode);

    return(ADI_OSAL_SUCCESS);
}


/*!
  ****************************************************************************
    @brief Returns the name of the currently executing thread


    @param[out] pszTaskName     - Pointer to the location to return the thread
                                  name
    @param[in]  nNumBytesToCopy - Number of bytes from the name to copy into
                                  the memory supplied as output argument

    @return ADI_OSAL_FAILED     - Unable to copy the task name
    @return ADI_OSAL_SUCCESS    - Successfully copied
    @return ADI_OSAL_BAD_HANDLE - The specified string pointer is invalid.

  *****************************************************************************/

ADI_OSAL_STATUS adi_osal_ThreadGetName(char_t *pszTaskName,
                                       uint32_t nNumBytesToCopy)
{
    char_t *pszThreadName = pszTaskName;
    char_t *pcTempStr;
    ADI_OSAL_THREAD_INFO_PTR hThreadNode;

#ifdef OSAL_DEBUG
    if (NULL == pszTaskName)
    {
        return(ADI_OSAL_BAD_HANDLE);
    }
#endif

    /* if we have less than 16 characters we don't identify the thread as
     * OSAL or not. Otherwise we do
     */
    if (nNumBytesToCopy >= 16u)
    {
        hThreadNode = (ADI_OSAL_THREAD_INFO_PTR) xTaskGetApplicationTaskTag(NULL);

        if ( _adi_osal_IsOSALThread(hThreadNode) )
        {
            pcTempStr ="OSAL_";
        }
        else
        {
            pcTempStr ="RTOS_";
        }
        memcpy(pszThreadName,pcTempStr,5u);
        pszThreadName += 5;
    }

    if (nNumBytesToCopy>= 11u)
    {
        *pszThreadName = '0';
        pszThreadName++;
        *pszThreadName = 'x';
        pszThreadName++;
    }

    if (nNumBytesToCopy >= 9u)
    {
        pcTempStr = pcTaskGetTaskName(NULL);

        while(*pcTempStr != '\0')
        {
            *pszThreadName = *pcTempStr;
            pszThreadName++;
            pcTempStr++;
        }

        *pszThreadName = '\0';
        return (ADI_OSAL_SUCCESS);
    }
    else
    {
        return (ADI_OSAL_FAILED);
    }

}


/*!
  ****************************************************************************
    @brief Returns the priority of a given thread.

    @param[in]  hThread      - handle to the thread that the API will return
                               the priority of.
    @param[out] pnThreadPrio - pointer to a location to write the thread
                               priority.

    @return ADI_OSAL_SUCCESS        -  If successfully returns the priority of the
                                       current thread
    @return ADI_OSAL_CALLER_ERROR   -  If function is invoked from an invalid
                                       location (i.e an ISR)
    @return ADI_OSAL_BAD_HANDLE     -  If the specified thread handle is invalid

    @note "*pnThreadPrio" will be set to "ADI_OSAL_INVALID_PRIORITY" if called from isr.

    @see adi_osal_ThreadSetPrio
*****************************************************************************/


ADI_OSAL_STATUS adi_osal_ThreadGetPrio(ADI_OSAL_THREAD_HANDLE const hThread, ADI_OSAL_PRIORITY *pnThreadPrio)
{
    ADI_OSAL_PRIORITY           nPrio;
    ADI_OSAL_THREAD_INFO_PTR hThreadNode = (ADI_OSAL_THREAD_INFO_PTR) hThread;

        /*  Not allowed from an ISR */
#ifdef OSAL_DEBUG
    if (CALLED_FROM_AN_ISR)
    {
        *pnThreadPrio = (ADI_OSAL_PRIORITY) ADI_OSAL_INVALID_PRIORITY;
           return (ADI_OSAL_CALLER_ERROR);
    }

    /* check validity of the handle */
    if ((NULL == hThreadNode) || (ADI_OSAL_INVALID_THREAD == hThread))
    {
        return (ADI_OSAL_BAD_HANDLE);
    }

    if ( !_adi_osal_IsOSALThread(hThreadNode) )
    {
        return (ADI_OSAL_BAD_HANDLE);
    }
#endif /* OSAL_DEBUG */

    nPrio  = (ADI_OSAL_PRIORITY) uxTaskPriorityGet(hThreadNode->pNativeThread);
    *pnThreadPrio = nPrio;

    return (ADI_OSAL_SUCCESS);
}


/*!
  ****************************************************************************
    @brief Changes the priority of the specified thread.

    @param[in] hThread       - Handle of the thread whose priority is to be
                               changed.
    @param[in] nNewPriority  - New desired priority.


    @return ADI_OSAL_SUCCESS      - If successfully changed the priority of the
                                    specified thread
    @return ADI_OSAL_BAD_PRIORITY - If the specified priority is invalid
    @return ADI_OSAL_CALLER_ERROR - If the function is invoked from an invalid
                                    location (i.e an ISR)
    @return ADI_OSAL_BAD_HANDLE   -  If the specified thread handle is invalid
    @see adi_osal_ThreadGetPrio
*****************************************************************************/


ADI_OSAL_STATUS adi_osal_ThreadSetPrio(ADI_OSAL_THREAD_HANDLE const hThread, ADI_OSAL_PRIORITY nNewPriority)
{
    UBaseType_t     nNewPrio;
    ADI_OSAL_STATUS eRetStatus;
    ADI_OSAL_THREAD_INFO_PTR hThreadNode = (ADI_OSAL_THREAD_INFO_PTR) hThread;

#ifdef OSAL_DEBUG
    /* check validity of the handle */
    if ((NULL == hThreadNode) || (ADI_OSAL_INVALID_THREAD == hThread))
    {
        return (ADI_OSAL_BAD_HANDLE);
    }

    if ( !_adi_osal_IsOSALThread(hThreadNode) )
    {
        return (ADI_OSAL_BAD_HANDLE);
    }
#endif

    /* FreeRTOS API cannot capture called from ISR Error, hence move it outside
     * OSAL_DEBUG */
    if (CALLED_FROM_AN_ISR)
    {
        return (ADI_OSAL_CALLER_ERROR);
    }

    /* FreeRTOS API doesn't check the prio when config assert is disabled hence move it outside
     * OSAL_DEBUG */
    if(nNewPriority >= configMAX_PRIORITIES)
    {
        return(ADI_OSAL_BAD_PRIORITY);
    }

    nNewPrio = (UBaseType_t) (nNewPriority + ADI_OSAL_FREERTOS_BASE_PRIO);

    vTaskPrioritySet(hThreadNode->pNativeThread, nNewPrio );

    eRetStatus = ADI_OSAL_SUCCESS;

    return(eRetStatus);
}


/*!
  ****************************************************************************
    @brief Stops the current thread running for the specified time in system
    ticks.

    @param[in] nTimeInTicks - Amount of time  to sleep in system ticks.

    @return ADI_OSAL_CALLER_ERROR - If function is invoked from an invalid
                                    location (i.e an ISR)
    @return ADI_OSAL_BAD_TIME     - If nTimeInTicks is more than ADI_OSAL_MAX_TIMEOUT
    @return ADI_OSAL_SUCCESS      - If successfully completed the "sleep" period

*****************************************************************************/
ADI_OSAL_STATUS  adi_osal_ThreadSleep(ADI_OSAL_TICKS nTimeInTicks)
{
    ADI_OSAL_STATUS eRetStatus = ADI_OSAL_SUCCESS;

#ifdef OSAL_DEBUG
    if (CALLED_FROM_AN_ISR)
    {
        return (ADI_OSAL_CALLER_ERROR);
    }

    if (CALLED_IN_SCHED_LOCK_REGION)
    {
        return (ADI_OSAL_CALLER_ERROR);
    }

    if(nTimeInTicks > ADI_OSAL_MAX_TIMEOUT)
    {
        return (ADI_OSAL_BAD_TIME);
    }

#endif /* OSAL_DEBUG */

    if (0u == nTimeInTicks)
    {
        return ADI_OSAL_SUCCESS;
    }

    vTaskDelay((TickType_t) nTimeInTicks);

    return(eRetStatus);
}
/* enable misra diagnostics as necessary */
/*! @cond */
#if defined ( __ICCARM__ )
_Pragma ("diag_default= Pm001,Pm002,Pm073,Pm088,Pm127,Pm128,Pm138,Pm139,Pm140,Pm141,Pm143")
#endif
/*! @endcond */
/*
**
** EOF:
**
*/
/*@}*/