/* $Revision: 30641 $
 * $Date: 2015-04-14 10:20:03 -0400 (Tue, 14 Apr 2015) $
******************************************************************************
Copyright (c), 2010-2013 - Analog Devices Inc. All Rights Reserved.
This software is PROPRIETARY & CONFIDENTIAL to Analog Devices, Inc.
and its licensors.
*****************************************************************************/

/*!
    @file adi_osal_freertos_tls.c

    Operating System Abstraction Layer - OSAL for FreeRTOS - TLS
    functions

*/
/** @addtogroup ADI_OSAL_TLS ADI OSAL TLS
 *  @{
 *
 *  This module contains the Thread Local Storage APIs for the FreeRTOS
 *  implementation of OSAL. Since FreeRTOS does not support TLS feature 
 *  the OSAL TLS APIs in this module return ADI_OSAL_OS_ERROR
 */
/*=============  I N C L U D E S   =============*/


#include "adi_osal.h"
#include "osal_common.h"

/* disable misra diagnostics as necessary
 *
 * Error[Pm008]: sections of code should not be 'commented out' 
 *               (MISRA C 2004 rule 2.4)
 *
 * Error[Pm128]: illegal implicit conversion from underlying MISRA type
 *                (MISRA C 2004 rule 10.1)
 */
/*! @cond */
#if defined ( __ICCARM__ )
_Pragma ("diag_suppress= Pm001,Pm002,Pm008,Pm128")
#endif
/*! @endcond */



/* Thread local storage numbers/keys consist of two values:
 *  - The signature, indicating that the key refers to a TLS slot 0xNNNNNNXX
 *  - The index of the slot, from 0 to Max Slots 0xXXXXXXNN
 */
#define TLS_SIGNATURE ((uint32_t)(0x544C5300u))

/* Masks used to extract either the signature component, or the slot index component
 * of a TLS Key.
 */
#define TLS_MASK_SIG ((uint32_t)(0xFFFFFF00u))
#define TLS_MASK_NUM ((uint32_t)(0x000000FFu))

/*=============  D A T A  =============*/

/*!
    @internal
    @var _adi_osal_gnNumSlots
         stores the number of TLS slots requested by the user during
         initialization.
    @endinternal
*/
uint32_t _adi_osal_gnNumSlots = ADI_OSAL_MAX_NUM_TLS_SLOTS;

/*!
    @internal
    @var  _adi_osal_gaTLSCallbacks
          Hold the callback structure for each Thread Local Storage Slot. The
          Callback are per slot, all threads that are using that slot are using
          the same callback.  Make the array the maximum supported size
          (ADI_OSAL_MAX_THREAD_SLOTS)
    @endinternal
*/
/*
static ADI_OSAL_TLS_CALLBACK_PTR _adi_osal_gaTLSCallbacks[ADI_OSAL_MAX_NUM_TLS_SLOTS];
*/

bool_t IsValidTLSKey(ADI_OSAL_TLS_SLOT_KEY key);

/*=============  C O D E  =============*/


#pragma inline
#if defined (__ECC__)
#pragma always_inline
#elif defined (__ICCARM__)
#pragma inline=forced
#endif
bool_t IsValidTLSKey(ADI_OSAL_TLS_SLOT_KEY key)
{
    return (TLS_SIGNATURE == (key & TLS_MASK_SIG));
}

/*!
  ****************************************************************************
    @brief Allocates a thread slot and returns the slot number

    Dummy function retuns ADI_OSAL_OS_ERROR.

    @param[out] pnThreadSlotKey       - Pointer to return the slot number if a
                                        free slot is found.  Must be populated
                                        with ADI_OSAL_TLS_UNALLOCATED.  If a
                                        valid slot is already present in the
                                        supplied address this API returns
                                        success.

    @param[in] pTerminateCallbackFunc - Pointer to a function that gets called 
                                        when the slot is freed.  Can be NULL
                                        if the callback function is not
                                        required.
  
    @return ADI_OSAL_OS_ERROR        - FreeRTOS does not support TLS feature.

*****************************************************************************/
ADI_OSAL_STATUS
adi_osal_ThreadSlotAcquire(ADI_OSAL_TLS_SLOT_KEY     *pnThreadSlotKey,
                           ADI_OSAL_TLS_CALLBACK_PTR pTerminateCallbackFunc)
{
    
    return (ADI_OSAL_OS_ERROR);
}



/*!
  ****************************************************************************
    @brief Frees the specified slot in the local storage buffer.

    @param[in] nThreadSlotKey     - slot which needs to be freed.

    @return ADI_OSAL_OS_ERROR        - FreeRTOS does not support TLS feature.

*****************************************************************************/
ADI_OSAL_STATUS
adi_osal_ThreadSlotRelease(ADI_OSAL_TLS_SLOT_KEY nThreadSlotKey)
{

    return(ADI_OSAL_OS_ERROR);
}


/*!
  ****************************************************************************
    @brief Stores the given value in the specified TLS slot.

    @param[out] nThreadSlotKey     - Slot key for the Thread Local Buffer in
                                     which "SlotValue" to be stored.
    @param[in] slotValue           - Value to be stored.

    @return ADI_OSAL_OS_ERROR      - FreeRTOS does not support TLS feature.
*****************************************************************************/
ADI_OSAL_STATUS
adi_osal_ThreadSlotSetValue(ADI_OSAL_TLS_SLOT_KEY nThreadSlotKey,
                            ADI_OSAL_SLOT_VALUE   slotValue)
{
    
    return(ADI_OSAL_OS_ERROR);
}



/*!
  ****************************************************************************

    @brief Gets a value for the specified TLS slot from the current thread.

    @param[in] nThreadSlotKey     - Slot key, from which the data needs
                                    to be retrieved.
    @param[out] pSlotValue        - Pointer to store the retrieved value from
                                    TLS.

    @return ADI_OSAL_OS_ERROR        - FreeRTOS does not support TLS feature.
*****************************************************************************/

ADI_OSAL_STATUS
adi_osal_ThreadSlotGetValue(ADI_OSAL_TLS_SLOT_KEY nThreadSlotKey,
                            ADI_OSAL_SLOT_VALUE   *pSlotValue)
{
    return(ADI_OSAL_OS_ERROR);
}

/* enable misra diagnostics as necessary */
/*! @cond */
#if defined ( __ICCARM__ )
_Pragma ("diag_default= Pm001,Pm002,Pm008,Pm128")
#endif
/*! @endcond */
/*
**
** EOF:
**
*/
/*@}*/
