/*
***************************************************************************
Copyright (c), 2009-2011 - Analog Devices Inc. All Rights Reserved.
This software is proprietary & confidential to Analog Devices, Inc.
and its licensors.
***************************************************************************

Title:   FreeRTOS-specific definitions

         This file contains the FreeRTOS specific defines.

*****************************************************************************/
/*!    @file osal_freertos.h */
#ifndef __OSAL_FREERTOS_H__
#define __OSAL_FREERTOS_H__



/*=============  I N C L U D E S   =============*/
#include "osal_misra.h"



/* Suppression for the FreeRTOS files */
ADI_OSAL_MISRA_SUPPRESS_ALL

#include <FreeRTOSConfig.h>
#include <FreeRTOS.h>
#include <task.h>
#include <queue.h>
#include <semphr.h>
#include <event_groups.h>

ADI_OSAL_MISRA_RESTORE_ALL

#include <adi_types.h>
#include "osal_common.h"
#include "adi_osal_arch_internal.h"

/*! @addtogroup ADI_OSAL_FreeRTOS ADI OSAL FreeRTOS
 *  @{
 */

/* disable misra diagnostics as necessary
 *
 * Error[Pm120]: the # and ## preprocessor operators should not be used
 *                (MISRA C 2004 rule 19.13)
 *
 * Error[Pm123]: there shall be no definition of objects or functions in a
 *                  header file (MISRA C 2004 rule 8.5)
 *
 * Error[Pm127]: a 'U' suffix shall be applied to all constants of 'unsigned' type
 *                (MISRA C 2004 rule 10.6)
 *
 * Error[Pm128]: illegal implicit conversion from underlying MISRA type
 *                (MISRA C 2004 rule 10.1)
 */
/*! @cond */
#if defined ( __ICCARM__ )
_Pragma ("diag_suppress= Pm001,Pm002,Pm120,Pm123,Pm127,Pm128")
#endif
/*! @endcond */

/*==============  D E F I N E S  ===============*/
#if defined(__STDC__)

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*!
  @def ADI_OSAL_PATCH_VER
  @details Patch version of  OSAL for FreeRTOS.  Please refer "adi_osal.h" for
  ADI_OSAL_MAJOR_VER and ADI_OSAL_MINOR_VER since these fields are
  also used for the code-base version
*/
#define ADI_OSAL_PATCH_VER              0u


#ifdef ADI_DEBUG
/* in case we ever want to turn on the checks in 'release' mode */
#define OSAL_DEBUG
#endif /* DEBUG */

/*!
  @def OSAL_INITIALIZED
  @details This preprocessor constant is used to indicate is the OSAL has been initialized.
   This is used instead of boolean flag in order to minimize the risk of match when
   the memory is uninitialized after processor boot
*/
#define OSAL_INITIALIZED                (0xAD10ul)

/*!
  @def INIT_COUNT_FOR_EMULATED_MUTEX
  @details Semaphores are used in place of Mutexes in this implementation
 */
#define INIT_COUNT_FOR_EMULATED_MUTEX   1u

/*!
  @def OSAL_MINIMUM_HEAP_SIZE
  @details Defines the minimum number of bytes that the user has to pass during OSAL
    Initialization to create the OSAL heap.  For now in FreeRTOS, there are no real
    requirements, so we just check that it is not 0
*/
#define OSAL_MINIMUM_HEAP_SIZE          ((uint32_t) 0u)


/*!
  @def CALLED_IN_SCHED_LOCK_REGION
  @details The scheduler and critical region locks counters can be used to verify if
  the user calls a function that can potentially create a deadlock situation
 */

#define CALLED_IN_SCHED_LOCK_REGION     (_adi_osal_IsSchedLocked())

/*!
  @def CALLED_BEFORE_OS_RUNNING
  @details The scheduler and critical region locks counters can be used to verify if
   the user calls a function that can potentially create a hang situation
 */

#define CALLED_BEFORE_OS_RUNNING       (_adi_osal_OsNotStarted())


/*!
  @def ADI_OSAL_FREERTOS_BASE_PRIO
  @details FreeRTOS doesn't reserve any priorities and the
    Idle thread priority in FreeRTOS is 0
*/
#define ADI_OSAL_FREERTOS_BASE_PRIO         tskIDLE_PRIORITY


/*!
  @def COMPATIBLE_OS_VERSION
  @details The current code has been developed and tested with FreeRTOS version 8.1.2
*/

#define COMPATIBLE_OS_VERSION           (0x812u)

/*!
  @def FREERTOS_NONBLOCKING_CALL
  @details For non blocking calls in FreeRTOS, No waiting
*/
#define FREERTOS_NONBLOCKING_CALL (0u)

/*!
  @def FREERTOS_MESSAGESIZE
  @details Message size in FreeRTOS is fixed as size of UBaseType_t
*/
#define  FREERTOS_MESSAGESIZE  (sizeof( UBaseType_t))


/*!
  @def EVENTFLAG_MAX_SIZE
  @details Event flag size is configured to 24 bit in FreeRTOS
*/
  #define EVENTFLAG_MAX_SIZE 0xFFFFFFu


/*!
  @def FREERTOS_SEMAPHORE_OBJ_SIZE
  @details Semaphore object size in FreeRTOS V8.1.2 is 76bytes.
  Queue_t is typedfed in tasks.c file and not the header
  file hence sizeof(Queue_t) cannot be used in the OSAL code.
  Thus defining the below macro
*/
#define  FREERTOS_SEMAPHORE_OBJ_SIZE   76u

/*!
  @def FREERTOS_EVENTGROUP_OBJ_SIZE
  @details Event group object size in FreeRTOS V8.1.2 is 28bytes.
  EventGroup_t is typedfed in event_groups.c file and hence
  sizeof(EventGroup_t) cannot be used in the OSAL code. Thus
  defining the below macro
*/
#define  FREERTOS_EVENTGROUP_OBJ_SIZE   28u


/*!
  @def FREERTOS_KERNEL_VERSION_NUM
  @details FreeRTOS Kernel Version Number
*/
#define FREERTOS_KERNEL_VERSION_NUMBER        tskKERNEL_VERSION_NUMBER

/*!
  @def FREERTOS_KERNEL_VERSION_MAJ
  @details FreeRTOS Kernel Version Major Number
*/
#define FREERTOS_KERNEL_VERSION_MAJOR         tskKERNEL_VERSION_MAJOR

/*!
  @def FREERTOS_KERNEL_VERSION_MIN
  @details FreeRTOS Kernel Version Minor Number
*/
#define FREERTOS_KERNEL_VERSION_MINOR         tskKERNEL_VERSION_MINOR

/*!
  @def FREERTOS_KERNEL_VERSION_BUI
  @details FreeRTOS Kernel Version Build Number
*/
#define FREERTOS_KERNEL_VERSION_BUILD         tskKERNEL_VERSION_BUILD

/*!
  @def FREERTOS_CRITICAL_ENABLE
  @details FreeRTOS OSAL Critical Region implementation
           when set to 1 it uses FreeRTOS method of enabling/disabling interrupts in Critical Region
           when set to 0 it uses native method of enabling/disabling interrupts in Critical Region
*/
#define FREERTOS_CRITICAL_ENABLE        0

static uint32_t _adi_osal_InterruptsDisable(void);
static void _adi_osal_InterruptsEnable(uint32_t previousState);

/*!
  ****************************************************************************
   @internal

   @fn _adi_osal_InterruptsDisable(void)

   @brief Interrupt Disable function

   @details This Function disables the interrupts from the OSAL

   Parameters:
   @return state - State of the interrupt mask register
   @endinternal

   @note In FreeRTOS, the critical region disables only those interrupts
   whose priority is less than the configMAX_SYSCALL_INTERRUPT_PRIORITY setting
   in FreeRTOSConfig.h file
*****************************************************************************/
#pragma inline
static uint32_t _adi_osal_InterruptsDisable(void)
{

    /* FreeRTOS way of implementing Critical Region */
#if FREERTOS_CRITICAL_ENABLE
    uint32_t state = ulPortSetInterruptMask();
#else
    uint32_t state = __get_PRIMASK();
    if(state == 0u)
    {
        __disable_irq();
    }
#endif

    return state;
}

/*!
  ****************************************************************************
   @internal

   @fn _adi_osal_InterruptsEnable(uint32_t previousState)

   @brief Interrupt Enable function

   @details This Function enables the interrupts from the OSAL

   Parameters:
   @param[in] previousState - Previous state of the interrupt mask register
   @endinternal

   @note In FreeRTOS, the critical region disables only those interrupts
   whose priority is less than the configMAX_SYSCALL_INTERRUPT_PRIORITY setting
   in FreeRTOSConfig.h file
*****************************************************************************/
#pragma inline
static void _adi_osal_InterruptsEnable(uint32_t previousState)
{
     /* FreeRTOS way of implementing Critical Region */
#if FREERTOS_CRITICAL_ENABLE
    vPortClearInterruptMask(previousState);
#else
    if(previousState == 0u)
    {
        __enable_irq();
    }
#endif

}
/*! @details Structure:  ADI_OSAL_THREAD_INFO
    This structure stores the thread information such as thread signature
    and thread handle
*/

typedef struct __AdiOsalThreadInfo
{
    /*! @var nThreadSignature
     * @details Stores the thread signature to determine if a pointer is an OSAL thread
     */
    uint32_t nThreadSignature;

    /*! @var pNativeThread
     * @details Stores the task  handle  of the created task with which it can be
     *  referenced
     */
    TaskHandle_t pNativeThread;
} ADI_OSAL_THREAD_INFO, *ADI_OSAL_THREAD_INFO_PTR;


/*!
  @def ADI_OSAL_THREAD_SIGNATURE
  @brief signature added to OSAL threads on creation
*/
#define ADI_OSAL_THREAD_SIGNATURE (0x41444954u)

/*!
  @def ADI_OSAL_INVALID_THREAD_SIGNATURE
  @brief signature added to OSAL threads on destruction
*/
#define ADI_OSAL_INVALID_THREAD_SIGNATURE (0x6e746872u)

/*!
  @def GetOSVersion()
  @brief Checks the Version of the RTOS used.
*/
#define GetOSVersion() (check_osversion())

uint32_t check_osversion( void );
bool_t _adi_osal_IsSchedLocked(void);
bool_t _adi_osal_OsNotStarted(void);
bool_t _adi_osal_IsOSALThread(ADI_OSAL_THREAD_INFO const *hThread);

/*! @details ADI_OSAL_MUTEX_INFO
    This structure provides a mapping between Mutex handle,thread handle and
    Acquisiton count for the mutexs.
*/
typedef struct __AdiOsalMutexInfo
{
    /*! @var hMutexID
     *  @details  Stores the mutex  handle return from FreeRTOS
     */
    SemaphoreHandle_t               hMutexID;

    /*! @var hOwnerThread
     *  @details Holds the task handle which acquires the mutex. This is a TaskHandle_t
     *  because we don't know whether the thread was created with OSAL or not.
     */
    TaskHandle_t         hOwnerThread;

    /*! @var nAcquisitionCount
     *  @details counter for mutex ownership.
     */
    uint32_t                nAcquisitionCount;

    /*! @var bDynAlloc
     *  @details indicates if this structure has been dynamically allocated.
     */
    bool_t                    bDynAlloc;

} ADI_OSAL_MUTEX_INFO, *ADI_OSAL_MUTEX_INFO_PTR;


/*! @fn check_osversion(void)
    @details This is function returns the FreeRTOS build version
*/
#pragma inline
uint32_t check_osversion( void )
{
    return ((FREERTOS_KERNEL_VERSION_MAJOR << (8))|(FREERTOS_KERNEL_VERSION_MINOR << (4))|(FREERTOS_KERNEL_VERSION_BUILD));
}


/*! @def ADI_OSAL_FREERTOS_MAX_TCB_SIZE
    @details This is the worst case TCB structure size (based on a single thread
    register) The 51 words have been computed using the worst case definition
    based on FreeRTOS v8.1.2 - Doesn't include configGENERATE_RUN_TIME_STATS
    and configUSE_NEWLIB_REENTRANT. Task Name length is ADI_OSAL_MAX_THREAD_NAME
*/
#define ADI_OSAL_FREERTOS_MAX_TCB_SIZE           ((uint32_t) (37u*4u ))


/*! @def COMPILE_TIME_ASSERT(condition,name)
 *  @brief Utility Macro to do compile time assertion
 */
#define COMPILE_TIME_ASSERT(condition, name) typedef uint8_t name##_failed[(condition)?1:-1]



/*=============  E X T E R N A L S  =============*/

/* Local global variables that are shared across files */
extern uint32_t _adi_osal_gnNumSlots;
extern uint32_t _adi_osal_gTLSAvailableSlots;
extern ADI_OSAL_TLS_CALLBACK_PTR _adi_osal_gaTLSCallbackTable[ADI_OSAL_MAX_NUM_TLS_SLOTS];
extern uint32_t _adi_osal_gnTickPeriod;
extern uint32_t _adi_osal_gkLowestPrio;
extern ADI_OSAL_THREAD_INFO _adi_osal_oStartupVirtualThread;
extern uint32_t _adi_osal_gnSchedulerLockCnt;
extern int32_t _adi_osal_gnCriticalRegionNestingCnt;

/*! @fn _adi_osal_IsSchedLocked(void)
 *  @details Scheduler Lock Status checks for taskSCHEDULER_SUSPENDED
 */
#pragma inline
bool_t _adi_osal_IsSchedLocked(void)
{
    return (taskSCHEDULER_SUSPENDED == xTaskGetSchedulerState());
}
/*! @fn _adi_osal_OsNotStarted(void)
 *  @details Scheduler Started Status checks for taskSCHEDULER_NOT_STARTED
 */
#pragma inline
bool_t _adi_osal_OsNotStarted(void)
{
    return (taskSCHEDULER_NOT_STARTED == xTaskGetSchedulerState());
}
/*! @fn _adi_osal_IsOSALThread(ADI_OSAL_THREAD_INFO const *hThread)
 *  @param[in] hThread - Thread Info
 *  @details Checks if Thread Signature is equal to ADI_OSAL_THREAD_SIGNATURE
 */
#pragma inline
bool_t _adi_osal_IsOSALThread(ADI_OSAL_THREAD_INFO const *hThread)
{
    return ( hThread->nThreadSignature == ADI_OSAL_THREAD_SIGNATURE);
}

#ifdef __cplusplus
}
#endif /* __cplusplus */

#else /* assembly language specific macros and declarations*/


#endif  /* if !defined(__ICCARM__) */

/* enable misra diagnostics as necessary */
/*! @cond */
#if defined ( __ICCARM__ )
_Pragma ("diag_default= Pm001,Pm002,Pm120,Pm123,Pm127,Pm128")
#endif
/*! @endcond */
#endif /*__OSAL_FREERTOS_H__ */

/*
**
** EOF:
**
*/
/*@}*/