/*********************************************************************************

Copyright(c) 2015 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
 * @file      adi_cpt_v1.c
 * @version: $Revision: 31412 $
 * @date:    $Date: 2015-06-11 16:38:18 -0400 (Thu, 11 Jun 2015) $
 *
 * @brief     This is the primary source file for the Capture Timer Service.
 *
 *            Contains the implementation of the APIs for the Capture Timer
 *            peripheral.
 *
 */

#include <sys/platform.h>
#include <stddef.h>
#include <string.h>
#include <assert.h>

/** @addtogroup CPT_Driver Capture Timer Service
 *  @{
 *
The processor includes three instants of Capture Timers
(CPTMR) to capture total ON time. Each Capture Timer captures
total ON time of the input signal between two leading
edges of the input trigger signal. Capture Timer inputs to all the
timers come from external pins and the input trigger signal
comes from Trigger Routing Unit (TRU).
   
The core of the timer is a 32-bit counter which is reset at leading
edge of the trigger and counts when the input signal level is
active. The total ON time of the input signal is captured from
the counter at the leading edge of the trigger pulse. Capture
Timer can generate data interrupts to the processor core at leading
edges of trigger pulses and status interrupts to indicate
counter overflow condition.
 */

#include <services/cpt/adi_cpt_v1.h>
#include "adi_cpt_def_v1.h"

#if defined(ADI_ADSP_CM40Z)
#include "adi_cpt_data_cm40z.c"
#else
#warning "This processor is not supported"
#endif

/**
 * @brief       Opens a capture timer device.
 *
 * @details     Opens a given capture timer device and returns the handle to the 
 *              timer which is used in other APIs.
 *
 * @param [in]  nTimerNum   Capture Timer number to open.
 * @param [in]  pMemory     Pointer to the memory which is required to operate the timer instance.
 *                          The size of the memory should be equal to #ADI_CPT_MEMORY bytes.
 * @param [in]  nMemSize    Size of the given memory in bytes.
 * @param [out] phTimer     Pointer to a location where the handle to the opened
 *                          timer is written.
 *
 * @return      Status
 * - #ADI_CPT_SUCCESS                   If successfully opened the given timer.
 * - #ADI_CPT_IN_USE                [D] If the given timer is in use.
 * - #ADI_CPT_INVALID_POINTER       [D] If any of the given pointer are invalid.
 * - #ADI_CPT_INSUFFICIENT_MEMORY   [D] If given memory is not sufficient to operate the timer
 *                                      instance.
 * - #ADI_CPT_INIT_FAILED               If timer service initialization failed.
 *
 * @sa adi_cpt_Close()
 */
ADI_CPT_RESULT  adi_cpt_Open (
    uint32_t                  nTimerNum,
    void *              const pMemory,
    uint32_t                  nMemSize,
    ADI_CPT_HANDLE     *const phTimer)
{
    ADI_CPT_DATA      *pTmrData = (ADI_CPT_DATA *)pMemory;
    ADI_CPT_DEVICE    *pDevice;

#ifdef ADI_DEBUG
    /* assert to validate the memory size constant */
    assert(ADI_CPT_MEMORY == sizeof(ADI_CPT_DATA));

    /* Check if the timer number is valid */
    if(nTimerNum >= ADI_CPT_NUM_DEVICES)
    {
        return ADI_CPT_INVALID_TIMERID;
    }

    /* Check if the pointer parameters are valid */
    if((pMemory == NULL) || (phTimer == NULL))
    {
        return ADI_CPT_INVALID_POINTER;
    }

    /* Check if the given memory is sufficient for the device operation */
    if(nMemSize < ADI_CPT_MEMORY)
    {
        return ADI_CPT_INSUFFICIENT_MEMORY;
    }
#endif /* ADI_DEBUG */

    /* Get the pointer to timer device */
    pDevice = &gCPTDevices[nTimerNum];

    /* Check if the timer is available */
    if(pDevice->pDevData != NULL)
    {
        return ADI_CPT_IN_USE;
    }

    /* Clear the given memory */
    memset(pMemory, 0, nMemSize);

    /* Clear the common registers bits for this timer */
    gCPTCommonRegs->CPT_RUNCL = (1u << nTimerNum);          /* W1C */
    gCPTCommonRegs->CPT_DATA_IMASKST = (1u << nTimerNum);   /* W1S */
    gCPTCommonRegs->CPT_STATUS_IMASKST = (1u << nTimerNum); /* W1S */
    gCPTCommonRegs->CPT_DATA_ILAT = (1u << nTimerNum);      /* W1C */
    gCPTCommonRegs->CPT_STATUS_ILAT = (1u << nTimerNum);    /* W1C */

    /* Clear the timer configuration */
    pDevice->pCPTRegs->CPT_CONFIG = 0u;
    
    /* Save the pointer to the device data, to mark the timer as in use */
    pDevice->pDevData = pTmrData;

    /* Register the capture timer status interrupt handler */
    if(adi_int_InstallHandler(pDevice->nStatusIntID,
                              StatusIntHandler,
                              pDevice,
                              true) != ADI_INT_SUCCESS)
    {
        pDevice->pDevData   =   NULL;
        
        return ADI_CPT_INT_FAILED;
    }

    /* Register the capture timer data interrupt handler */
    if(adi_int_InstallHandler(pDevice->nDataIntID,
                              DataIntHandler,
                              pDevice,
                              true) != ADI_INT_SUCCESS)
    {
        pDevice->pDevData   =   NULL;
        
        return ADI_CPT_INT_FAILED;
    }

    /* Return the timer handle */
    *phTimer = (ADI_CPT_HANDLE) pDevice;

    return ADI_CPT_SUCCESS;
}

/**
 * @brief       Closes the given capture timer.
 *
 * @details     Once the timer is closed the handle to the timer will be
 *              invalid. It cannot be used to access any of the timer APIs.
 *
 *              The API stops the capture timer, masks all interrupts and
 *              unregisters interrupt handlers.
 *
 * @param [in]  hTimer      Handle to the capture timer.
 *
 * @return      Status
 * - #ADI_CPT_SUCCESS               If successfully closed the given timer.
 * - #ADI_CPT_INVALID_HANDLE    [D] If the given timer handle is invalid.
 * - #ADI_CPT_INT_FAILED            Failed to uninstall the interrupt handler.
 *
 * @sa adi_cpt_Open()
 */
ADI_CPT_RESULT  adi_cpt_Close (
    ADI_CPT_HANDLE    const hTimer)
{
    ADI_CPT_DEVICE  *pDevice = (ADI_CPT_DEVICE *) hTimer;
    uint32_t           nTimerNum;

#ifdef ADI_DEBUG
    ADI_CPT_RESULT  eResult;

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_CPT_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    nTimerNum = pDevice->nTimerID;

    /* Clear the common registers bits for this timer */
    gCPTCommonRegs->CPT_RUNCL = (1u << nTimerNum);          /* W1C */
    gCPTCommonRegs->CPT_DATA_IMASKST = (1u << nTimerNum);   /* W1S */
    gCPTCommonRegs->CPT_STATUS_IMASKST = (1u << nTimerNum); /* W1S */
    gCPTCommonRegs->CPT_DATA_ILAT = (1u << nTimerNum);      /* W1C */
    gCPTCommonRegs->CPT_STATUS_ILAT = (1u << nTimerNum);    /* W1C */

    /* Uninstall the capture timer status interrupt handler */
    if(adi_int_UninstallHandler(pDevice->nStatusIntID) != ADI_INT_SUCCESS)
    {
        return ADI_CPT_INT_FAILED;
    }

    /* Uninstall the capture timer data interrupt handler */
    if(adi_int_UninstallHandler(pDevice->nDataIntID) != ADI_INT_SUCCESS)
    {
        return ADI_CPT_INT_FAILED;
    }

    pDevice->pDevData = NULL;

    return ADI_CPT_SUCCESS;
}

/**
 * @brief       Registers a user callback function with the driver.
 *
 * @details     Notifies the application when a capture timer count rollover or 
 *              data interrupt occurs.
 *
 * @param [in]  hTimer              Handle to the capture timer device.
 *
 * @param [in]  pfCallback          Pointer to the callback function. The callback function
 *                                  has the prototype
 *                                  void callback(void *pCBParam, uint32_t nEvent, void *pArg)
 *
 *                                  To unregister a callback pass a NULL pointer to the callback
 *                                  function.
 *
 * @param [in]  pCBParam            Callback parameter which will be returned back to the
 *                                  application when the callback function is called.
 *
 * @return      Status
 *  - #ADI_CPT_SUCCESS                If successfully registered a callback.
 *
 *  - #ADI_CPT_INVALID_HANDLE     [D] If the given capture timer handle is invalid.
 *
 */
ADI_CPT_RESULT adi_cpt_RegisterCallback(
    ADI_CPT_HANDLE          const hTimer,
    ADI_CALLBACK            const pfCallback,
    void                   *const pCBParam)
{
    ADI_CPT_DEVICE  *pDevice = (ADI_CPT_DEVICE *) hTimer;

#ifdef ADI_DEBUG
    ADI_CPT_RESULT  eResult;

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_CPT_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* Store the given callback and callback parameter */
    pDevice->pDevData->pfCallback = pfCallback;
    pDevice->pDevData->pCBParam = pCBParam;
    
    return ADI_CPT_SUCCESS;
}

/**
 * @brief       Configure and enable the capture timer input pin.
 *
 * @details     Configure the capture timer input selection and polarity.  Each 
 *              capture timer has a input or aux selection. The polarity
 *              is set to inverted or not inverted.
 *
 * @note:       For ADSP-CM40x the AUX input is not used.
 *
 * @param [in]  hTimer      Handle to the capture timer.
 * @param [in]  bAuxSel     If true, the aux pin is selected for capture.
 *                          If false, the input pin is selected for capture.
 * @param [in]  bPolInvert  If true, the polarity is set to inverted 
 *                          (active low/falling edge as leading edge)
 *                          If false, the polarity is set to not inverted.
 *                          (active high/rising edge as leading edge)
 * @param [in]  bEnable     If true, the capture timer input is enabled.
 *                          If false, the capture timer input is disabled.
 *
 * @return      Status
 * - #ADI_CPT_SUCCESS               If successfully configured the input pin.
 * - #ADI_CPT_INVALID_HANDLE    [D] If the given timer handle is invalid.
 */
ADI_CPT_RESULT  adi_cpt_InputConfig(
    ADI_CPT_HANDLE          const hTimer,
    bool                          bAuxSel,
    bool                          bPolInvert,
    bool                          bEnable)
{
    ADI_CPT_DEVICE  *pDevice = (ADI_CPT_DEVICE *) hTimer;
    uint8_t regData;
    
#ifdef ADI_DEBUG
    ADI_CPT_RESULT  eResult;

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_CPT_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
  
    /* Capture timer config */
    regData = pDevice->pCPTRegs->CPT_CONFIG;

    /* clear config bits */
    regData &= ~(BITM_CPTMR_CFG_TINSEL | BITM_CPTMR_CFG_TINPOL | 
                 BITM_CPTMR_CFG_TIN_INEN | BITM_CPTMR_CFG_AUX_TIN_INEN);
    
    if (bAuxSel)
    {
        regData |= (1u << BITP_CPTMR_CFG_TINSEL );
    }
    
    if (bPolInvert)
    {
        regData |= (1u << BITP_CPTMR_CFG_TINPOL);
    }
        
    if (bEnable)
    {
        if (bAuxSel)
        {
            regData |= (1u << BITP_CPTMR_CFG_AUX_TIN_INEN );
        }
        else
        {
            regData |= (1u << BITP_CPTMR_CFG_TIN_INEN );
        }
    }
    
    /* Capture timer config */
    pDevice->pCPTRegs->CPT_CONFIG = regData;
      
    return ADI_CPT_SUCCESS;
}

/**
 * @brief       Configure and enable the capture timer trigger pin.
 *
 * @details     Configure the capture timer trigger selection and polarity.  Each 
 *              capture timer has a trigger or aux trigger selection. The polarity
 *              is set to inverted or not inverted.
 *
 * @note:       For ADSP-CM40x the trigger inputs are from the Trigger Routing Unit (TRU) 
 *              and AUX trigger is not used.
 *
 * @param [in]  hTimer      Handle to the capture timer.
 * @param [in]  bAuxSel     If true, the aux trigger pin is selected.
 *                          If false, the trigger pin is selected.
 * @param [in]  bPolInvert  If true, the polarity is set to inverted.
 *                          (active low/falling edge as leading edge)
 *                          If false, the polarity is set to not inverted.
 *                          (active high/rising edge as leading edge)
 * @param [in]  bEnable     If true, the capture timer trigger input is enabled.
 *                          If false, the capture timer trigger input is disabled.
 *
 * @return      Status
 * - #ADI_CPT_SUCCESS               If successfully configured the trigger pin.
 * - #ADI_CPT_INVALID_HANDLE    [D] If the given timer handle is invalid.
 */
ADI_CPT_RESULT  adi_cpt_TriggerConfig(
    ADI_CPT_HANDLE          const hTimer,
    bool                          bAuxSel,
    bool                          bPolInvert,    
    bool                          bEnable)
{
    ADI_CPT_DEVICE  *pDevice = (ADI_CPT_DEVICE *) hTimer;
    uint8_t regData;
    
#ifdef ADI_DEBUG
    ADI_CPT_RESULT  eResult;

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_CPT_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
  
    /* Capture timer config */
    regData = pDevice->pCPTRegs->CPT_CONFIG;

    /* clear config bits */
    regData &= ~(BITM_CPTMR_CFG_TRIGSEL | BITM_CPTMR_CFG_TRIGPOL | 
                 BITM_CPTMR_CFG_TRIG_INEN | BITM_CPTMR_CFG_AUX_TRIG_INEN);
    
    if (bAuxSel)
    {
        regData |= (1u << BITP_CPTMR_CFG_TRIGSEL );
    }
    
    if (bPolInvert)
    {
        regData |= (1u << BITP_CPTMR_CFG_TRIGPOL);
    }
        
    if (bEnable)
    {
        if (bAuxSel)
        {
            regData |= (1u << BITP_CPTMR_CFG_AUX_TRIG_INEN );
        }
        else
        {
            regData |= (1u << BITP_CPTMR_CFG_TRIG_INEN );
        }
    }
    
    /* Capture timer config */
    pDevice->pCPTRegs->CPT_CONFIG = regData;
      
    return ADI_CPT_SUCCESS;
}

/**
 * @brief       Gets the capture timer count.
 *
 * @details     The core of the timer is a 32-bit counter which is reset at leading
 *              edge of the trigger and counts when the input signal level is
 *              active.
 *
 *              A callback status interrupt indicates counter overflow condition.
 *
 *
 * @param [in]  hTimer              Handle to the capture timer.
 * @param [out] pCount              The capture timer count.
 *
 * @return      Status
 * - #ADI_CPT_SUCCESS               If successfully got the timer count.
 * - #ADI_CPT_INVALID_HANDLE    [D] If the given timer handle is invalid.
 * 
 * @sa adi_cpt_RegisterCallback()
 * @sa adi_cpt_Start()
 */
ADI_CPT_RESULT  adi_cpt_GetCount(
    ADI_CPT_HANDLE          const hTimer,
    uint32_t                     *pCount)
{
    ADI_CPT_DEVICE  *pDevice = (ADI_CPT_DEVICE *) hTimer;
    
#ifdef ADI_DEBUG
    ADI_CPT_RESULT  eResult;

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_CPT_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
  
    /* Capture timer counter */
    *pCount = pDevice->pCPTRegs->CPT_COUNTER;

    return ADI_CPT_SUCCESS;
}

/**
 * @brief       Gets the total ON time count of the input signal.
 *
 * @details     Each Capture Timer captures total ON time of the input signal 
 *              between two leading edges of the input trigger signal.
 *
 *
 * @param [in]  hTimer              Handle to the capture timer.
 * @param [out] pCount              The the total ON time count of the input waveform. 
 *                                  The ON time is measured in SCLK units.
 *
 * @return      Status
 * - #ADI_CPT_SUCCESS               If successfully got the total ON time.
 * - #ADI_CPT_INVALID_HANDLE    [D] If the given timer handle is invalid.
 * 
 * @sa adi_cpt_RegisterCallback()
 * @sa adi_cpt_Start()
 */
ADI_CPT_RESULT  adi_cpt_GetOnTimeCount(
    ADI_CPT_HANDLE          const hTimer,
    uint32_t                     *pCount)
{
    ADI_CPT_DEVICE  *pDevice = (ADI_CPT_DEVICE *) hTimer;
    
#ifdef ADI_DEBUG
    ADI_CPT_RESULT  eResult;

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_CPT_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
  
    /* Capture timer On Time */
    *pCount = pDevice->pCPTRegs->CPT_TON;

    return ADI_CPT_SUCCESS;
}

/* 
 * capture timer common API's 
 */

/**
 * @brief       Start the capture timers.
 *
 * @details     Start a group of capture timers.
 *
 * @param [in]  nGroupMask          Select one or more timers to start.  Multiple
 *                                  timers are specified by OR'ing the #ADI_CPT_MASK
 *                                  e.g. ADI_CPT_TMR0|ADI_CPT_TMR1|ADI_CPT_TMR2
 *
 * @return      Status
 * - #ADI_CPT_SUCCESS               If successfully started the capture timers.
 * 
 * @sa adi_cpt_Stop()
 * @sa adi_cpt_GetCount()
 * @sa adi_cpt_GetOnTimeCount()
 */
ADI_CPT_RESULT  adi_cpt_Start(
    uint16_t             nGroupMask)
{
    /* start capture based on mask */
    gCPTCommonRegs->CPT_RUNST = nGroupMask;

    return ADI_CPT_SUCCESS;
}

/**
 * @brief       Stop the capture timers.
 *
 * @details     Stop a group of capture timers.
 *
 * @param [in]  nGroupMask          Select one or more timers to stop.  Multiple
 *                                  timers are specified by OR'ing the #ADI_CPT_MASK
 *                                  e.g. ADI_CPT_TMR0|ADI_CPT_TMR1|ADI_CPT_TMR2
 *
 * @return      Status
 * - #ADI_CPT_SUCCESS               If successfully stopped the capture timers.
 * 
 * @sa adi_cpt_Start()
 * @sa adi_cpt_GetCount()
 * @sa adi_cpt_GetOnTimeCount()
 */
ADI_CPT_RESULT  adi_cpt_Stop(
    uint16_t             nGroupMask)
{
    /* stop capture based on mask */
    gCPTCommonRegs->CPT_RUNCL = nGroupMask;

    return ADI_CPT_SUCCESS;
}

/**
 * @brief       Enable/Disable the data interrupt.
 *
 * @details     The data interrupt is used to detect the leading edge of the trigger pulse.
 *              The #ADI_CPT_EVENT_TRIGGER_EDGE event is passed to the callback when
 *              a data interrupt occurs.
 *
 * @param [in]  nGroupMask          Select one or more timers to enable/disable the data
 *                                  interrupt.  Multiple timers are specified by OR'ing 
 *                                  the #ADI_CPT_MASK
 *                                  e.g. ADI_CPT_TMR0|ADI_CPT_TMR1|ADI_CPT_TMR2
 *
 * @param [in]  bEnable             True to enable the data interrupt.
 *                                  False to disable the data interrupt.
 *
 * @return      Status
 * - #ADI_CPT_SUCCESS               If successfully Enable/Disable the data interrupt.
 * 
 * @sa adi_cpt_RegisterCallback()
 */
ADI_CPT_RESULT  adi_cpt_EnableDataInt(
    uint16_t             nGroupMask,
    bool                 bEnable)
{
    if (bEnable)
    {
        /* enable data interrupt request (clear mask) */
        gCPTCommonRegs->CPT_DATA_IMASKCL = nGroupMask;
    }
    else
    {
        /* disable data interrupt request (set mask) */
        gCPTCommonRegs->CPT_DATA_IMASKST = nGroupMask;
    }

    return ADI_CPT_SUCCESS;
}

/**
 * @brief       Enable/Disable the status interrupt.
 *
 * @details     The status interrupt is used to detect counter overflow.  At overflow 
 *              the counter resets to 1. After 2^32 - 1 clocks the counter overflows.
 *              The #ADI_CPT_EVENT_COUNT_OVERFLOW event is passed to the callback when
 *              a status interrupt occurs.
 *
 * @param [in]  nGroupMask          Select one or more timers to enable/disable the status
 *                                  interrupt.  Multiple timers are specified by OR'ing 
 *                                  the #ADI_CPT_MASK
 *                                  e.g. ADI_CPT_TMR0|ADI_CPT_TMR1|ADI_CPT_TMR2
 *
 * @param [in]  bEnable             True to enable the status interrupt.
 *                                  False to disable the status interrupt.
 *
 * @return      Status
 * - #ADI_CPT_SUCCESS               If successfully Enable/Disable the status interrupt.
 * 
 * @sa adi_cpt_RegisterCallback()
 */
ADI_CPT_RESULT  adi_cpt_EnableStatusInt(
    uint16_t             nGroupMask,
    bool                 bEnable)
{
    if (bEnable)
    {
        /* enable data interrupt request (clear mask) */
        gCPTCommonRegs->CPT_STATUS_IMASKCL = nGroupMask;
    }
    else
    {
        /* disable data interrupt request (set mask) */
        gCPTCommonRegs->CPT_STATUS_IMASKST = nGroupMask;
    }

    return ADI_CPT_SUCCESS;
}

/* Status interrupt handler */
static void StatusIntHandler(
    uint32_t            IID,
    void               *pCBParam
)
{
    ADI_CPT_DEVICE  *pDevice = (ADI_CPT_DEVICE *) pCBParam;

    /* Clear the status interrupt latch (W1C) */
    gCPTCommonRegs->CPT_STATUS_ILAT = (1u << pDevice->nTimerID);    /* W1C */

    if (pDevice->pDevData->pfCallback != NULL)
    {
        /* Call the application handler */
        pDevice->pDevData->pfCallback(pDevice->pDevData->pCBParam,
                                  (uint32_t)ADI_CPT_EVENT_COUNT_OVERFLOW,
                                  NULL);
    }
    
    return;
}

/* Data interrupt handler */
static void DataIntHandler(
    uint32_t            IID,
    void               *pCBParam
)
{
    ADI_CPT_DEVICE  *pDevice = (ADI_CPT_DEVICE *) pCBParam;

    /* Clear the data interrupt latch (W1C) */
    gCPTCommonRegs->CPT_DATA_ILAT = (1u << pDevice->nTimerID);    /* W1C */

    if (pDevice->pDevData->pfCallback != NULL)
    {
        /* Call the application handler */
        pDevice->pDevData->pfCallback(pDevice->pDevData->pCBParam,
                                  (uint32_t)ADI_CPT_EVENT_TRIGGER_EDGE,
                                  NULL);
    }
    
    return;
}

#ifdef ADI_DEBUG
/*
 * Verify that the handle is valid
 */
static ADI_CPT_RESULT ValidateDeviceHandle (
    ADI_CPT_DEVICE     *const pDevice)
{
    uint32_t nTimerNum;

    /* Check if the given pointer is valid */
    if(pDevice == NULL)
    {
        return ADI_CPT_INVALID_POINTER;
    }

    /* For all the timers */
    for( nTimerNum = 0u; nTimerNum < ADI_CPT_NUM_DEVICES; nTimerNum++)
    {
        /* Check if the given handle matches with this timer */
        if( pDevice == &gCPTDevices[nTimerNum])
        {
            return ADI_CPT_SUCCESS;
        }
    }

    return ADI_CPT_INVALID_HANDLE;
}
#endif

/*@}*/
