/*********************************************************************************

Copyright(c) 2013-2015 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
 * @file      adi_tru.c
 * @brief     Trigger Routing Unit Manager.
 * @version:  $Revision: 31623 $
 * @date:     $Date: 2015-07-06 00:49:23 -0400 (Mon, 06 Jul 2015) $
 *
 * @details
 *            This is the the primary source file for Trigger Routing Unit Manager
 *
 */

/** @addtogroup TRU_Manager Trigger Routing Unit (TRU) Manager
 *  @{
 */

/*=============  I N C L U D E S   =============*/
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be longer than 31 characters")
#endif

/* TRU Manager includes */
#include <services/tru/adi_tru.h>
/* Include header file with data structure definitions used in TRU manager implementation */
#include "adi_tru_def.h"

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

/*=============  L O C A L    F U N C T I O N    P R O T O T Y P E S =============*/

/*=============  D E B U G    F U N C T I O N    P R O T O T Y P E S  =============*/

/* IF (Debug mode enabled) */
#if defined (ADI_DEBUG)

#endif /* ADI_DEBUG */

/*=============  T R U    I M P L E M E N T A T I O N    S O U R C E    F I L E S  =============*/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_6:"")
#pragma diag(suppress:misra_rule_5_7:"")
#pragma diag(suppress:misra_rule_8_7:"Allow objects to be defined in a common file")
#pragma diag(suppress:misra_rule_11_4:"")
#pragma diag(suppress:misra_rule_14_7:"Allow multiple exit for a function")
#pragma diag(suppress:misra_rule_16_7:"")
#endif /* _MISRA_RULES */

/* Processor specific data for TRU Service */
/* IF (ADSP-BF606/607/608/609) */
#if defined( __ADSPBF609_FAMILY__)
/* ADSP-BF60x Processor specific data for TRU manager */
#include "adi_tru_data_bf60x.c"
/* ELSE (ADSP-CM402/CM403/CM407/CM408 processors) */
#elif defined (ADI_ADSP_CM40Z)
/* ADSP-CM40z Processor specific data for TRU manager */
#include "adi_tru_data_cm40z.c"
/* ELSE (ADSP-CM41X processors) */
#elif defined (ADI_ADSP_CM41X)
/* ADSP-CM41x Processor specific data for TRU manager */
#include "adi_tru_data_cm41x.c"
#elif defined (__ADSPBF707_FAMILY__)
/* ADSP-BF70x Processor specific data for TRU manager */
#include "adi_tru_data_bf70x.c"
#elif defined(__ADSPSC589_FAMILY__)
/* ADSP-SC58x Processor specific data for TRU manager */
#include "adi_tru_data_sc58x.c"
/* ELSE */
#else
#error "TRU Manager not available for this processor"
#endif

/*=============  D E B U G    F U N C T I O N    P R O T O T Y P E S  =============*/

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

/* Validates TRU device handle */
static ADI_TRU_RESULT ValidateTruHandle (ADI_TRU_HANDLE const  hDevice);

#endif /* ADI_DEBUG */

/*=============  C O D E  =============*/

/*=============  L O C A L    F U N C T I O N S  =============*/

/*=============  D E B U G   F U N C T I O N S =============*/

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

/* Validates TRU device handle */
static ADI_TRU_RESULT ValidateTruHandle (ADI_TRU_HANDLE const  hDevice)
{
    /* Loop variable */
    uint32_t    i;

    /* FOR (All CRC Device instances) */
    for (i = 0u; i < (sizeof (TruInfo) / sizeof (TruInfo[0])); i++)
    {
        /* IF (TRU handle is valid) */
        if (hDevice == &TruInfo[i])
        {
            /* Channel valid, return success */
            return (ADI_TRU_SUCCESS);
        }
    }

    /* Device ID invalid, return failure */
    return (ADI_TRU_FAILURE);
}

#endif /* ADI_DEBUG */

/*=============  P U B L I C   F U N C T I O N S =============*/

/**
 * @brief Opens a Trigger Routing Unit. This API is deprecated.
 *
 * @param [in]  DeviceNumber    Trigger Routing Unit Device number to open.
 * @param [in]  pMemory         Pointer to ADI_TRU_REQ_MEMORY-sized application memory
 *                              in which the TRU device driver will store driver data.
 * @param [in]  MemorySize      Size of the buffer to which "pMemory" points, i.e., ADI_TRU_REQ_MEMORY.
 * @param [in]  phDevice        Address to application memory in which the TRU device handle will be stored on success.
 *                              The TRU device handle is required to be passed to all other TRU APIs.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully opened trigger routing unit.
 *          - #ADI_TRU_IN_USE: TRU device is already in use.
 *          - #ADI_TRU_BAD_DEVNUM: Invalid TRU device ID.
 * @note - This API is deprecated. The recommended TRU sequence is to use the
 *         adi_tru_Init/adi_tru_UnInit APIs
 * @sa adi_tru_Init adi_tru_UnInit
 *
 */
ADI_TRU_RESULT  adi_tru_Open(
    uint32_t            DeviceNumber,
    void *const         pMemory,
    uint32_t            MemorySize,
    ADI_TRU_HANDLE      *phDevice)
{
    /* Return Code */
    ADI_TRU_RESULT      eResult = ADI_TRU_SUCCESS;

    /* Memory to handle TRU */
    ADI_TRU_DEV         *pDevice = (ADI_TRU_DEV *) pMemory;

    /* verify device number (only one device instance for TRU so far...) */
    if (0u != DeviceNumber)
    {
    	return (ADI_TRU_BAD_DEVNUM);
    }

    /* IF (TRU device is already in use) */
    if (TruInfo[DeviceNumber].pDevice != NULL)
    {
        /* Report failure as TRU is already in use */
        eResult = ADI_TRU_IN_USE;
    }

    /* IF (TRU device is available for use) */
    if (eResult == ADI_TRU_SUCCESS)
    {
        /* Entering critical region, disable interrupts */
        adi_osal_EnterCriticalRegion();

        /* Save the memory address allocated to handle this device channel */
        TruInfo[DeviceNumber].pDevice = pDevice;
        /* Save the device number */
        pDevice->DeviceNumber = (uint8_t) DeviceNumber;
        /* Soft-Reset TRU */
        TruInfo[DeviceNumber].pReg->GlobalControl |= BITM_TRU_GCTL_RESET;

        /* Re-enable interrupts */
	    adi_osal_ExitCriticalRegion();

	    /* Pass a valid handle to this TRU device */
        *phDevice = (ADI_TRU_HANDLE) &TruInfo[DeviceNumber];
	}

    return (eResult);
}

/**
 * @brief Closes a Trigger Routing Unit. This API is deprecated.
 *
 * @param [in]  hDevice     Handle to the TRU device to be closed.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully closed trigger routing unit.
 *          - #ADI_TRU_BAD_HANDLE: TRU device handle is invalid.
 *
 * @note - This API is deprecated. The recommended TRU sequence is to use the
 *         adi_tru_Init/adi_tru_UnInit APIs
 * @sa adi_tru_Init adi_tru_UnInit
 *
 */
ADI_TRU_RESULT  adi_tru_Close(
    ADI_TRU_HANDLE      hDevice)
{
    /* Memory to handle TRU */
    ADI_TRU_INFO    *pDevInfo = (ADI_TRU_INFO *) hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (TRU device handle is invalid) */
    if (ValidateTruHandle (hDevice) != ADI_TRU_SUCCESS)
    {
        return (ADI_TRU_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* Disable TRU */
    adi_tru_Enable (hDevice, false);
    /* Clear the device handle */
    pDevInfo->pDevice = NULL;

    return (ADI_TRU_SUCCESS);
}

/**
 * @brief Routes/Maps a single trigger master to a single slave. This API is deprecated.
 *
 * @details This API will make the given Slave to wait (listen) for the trigger from the
 *          given master. Multiple slaves can wait for the trigger from the single master.
 *          To achieve multiple slave routings to a single master, this API may be called
 *          multiple times with different Slave IDs, but with same Master ID.
 *
 * @param [in]  hDevice             Handle to the TRU device to work on.
 * @param [in]  TriggerSlave        Trigger Slave ID.
 * @param [in]  TriggerMaster       Trigger Master ID.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully mapped a trigger master to slave.
 *          - #ADI_TRU_BAD_HANDLE: TRU device handle is invalid.
 *
 * @note
 *      The Trigger Master and Slave IDs are defined in processor definition file (defxxxx.h) \n
 *      defBF60x.h - Definition file for ADSP-BF60x Processor family \n
 *      defBF70x.h - Definition file for ADSP-BF70x Processor family \n
 *      defCM40z.h - Definition file for ADSP-CM40z Processor family \n
 *      defCM41x.h - Definition file for ADSP-CM41x Processor family \n
 *      defSC589.h - Definition file for ADSP-SC58x Processor family \n
 * @note - This API is deprecated. The recommended TRU sequence is to use the
 *         adi_tru_ConfigureSlave API
 * @sa adi_tru_ConfigureSlave
 *
 */
ADI_TRU_RESULT  adi_tru_TriggerRoute(
    ADI_TRU_HANDLE      hDevice,
    uint32_t            TriggerSlave,
    uint32_t            TriggerMaster)
{
    /* Memory to handle TRU */
    ADI_TRU_INFO        *pDevInfo = (ADI_TRU_INFO *)hDevice;
    /* Pointer to Slave Select registers */
    volatile uint32_t   (*TriggerSlaveReg)[] = (volatile uint32_t(*)[])pDevInfo->pSSReg;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (TRU device handle is invalid) */
    if (ValidateTruHandle (hDevice) != ADI_TRU_SUCCESS)
    {
        return (ADI_TRU_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* Map trigger master to slave */
    (*TriggerSlaveReg)[TriggerSlave] = TriggerMaster;

    return (ADI_TRU_SUCCESS);
}

/**
 * @brief Enables/Disables Trigger Routing Unit. This API is deprecated.
 *
 * Parameters:
 *
 * @param [in]  hDevice     Handle to the TRU device
 * @param [in]  bEnable     'true' to enable Trigger routing, 'false' to disable.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully configured TRU.
 *          - #ADI_TRU_BAD_HANDLE: TRU device handle is invalid.
 *
 * @note - This API is deprecated. The recommended TRU sequence is to use the
 *         adi_tru_Init/adi_tru_UnInit APIs
 * @sa adi_tru_Init adi_tru_UnInit
 *
 */
ADI_TRU_RESULT  adi_tru_Enable(
    ADI_TRU_HANDLE      hDevice,
    bool                bEnable)
{
    /* Memory to handle TRU */
    ADI_TRU_INFO        *pDevInfo = (ADI_TRU_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (TRU device handle is invalid) */
    if (ValidateTruHandle (hDevice) != ADI_TRU_SUCCESS)
    {
        return (ADI_TRU_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* IF (Enable TRU) */
    if (bEnable)
    {
        pDevInfo->pReg->GlobalControl |= BITM_TRU_GCTL_EN;
    }
    /* ELSE (Disable TRU) */
    else
    {
        pDevInfo->pReg->GlobalControl &= ~BITM_TRU_GCTL_EN;
    }

    return (ADI_TRU_SUCCESS);
}

/**
 * @brief Generate a Master Trigger(s). This API is deprecated.
 *
 * @details This API writes the Master Trigger Register (MTR). Any master trigger
 *          (including the software based triggers) can be generated by calling this API.
 *          The trigger will be generated only if the TRU is enabled by calling #adi_tru_Enable().
 *          The trigger will route to the currently routed trigger slave(s),
 *          as configured by #adi_tru_TriggerRoute().
 *
 * @param [in]  hDevice             Handle to the TRU device
 * @param [in]  MasterTriggerVal    Master Trigger Register (MTR) value to write.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully configured TRU.
 *          - #ADI_TRU_BAD_HANDLE: TRU device handle is invalid.
 *
 * @note - This API is deprecated. The recommended TRU sequence is to use the
 *         adi_tru_RaiseTriggerMaster API
 * @sa adi_tru_RaiseTriggerMaster
 */
ADI_TRU_RESULT adi_tru_ConfigTriggerMaster(
    ADI_TRU_HANDLE      hDevice,
    uint32_t            MasterTriggerVal)
{
    /* Memory to handle TRU */
    ADI_TRU_INFO        *pDevInfo = (ADI_TRU_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (TRU device handle is invalid) */
    if (ValidateTruHandle (hDevice) != ADI_TRU_SUCCESS)
    {
        return (ADI_TRU_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* Configure the Trigger Master register */
    pDevInfo->pReg->MasterTrigger = MasterTriggerVal;

    return (ADI_TRU_SUCCESS);
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

/*****/

/*@}*/

