/*********************************************************************************

Copyright(c) 2013-2015 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
 * @file      adi_tru_V2.c
 * @brief     Trigger Routing Unit Manager.
 * @version:  $Revision: 31623 $
 * @date:     $Date: 2015-07-06 00:49:23 -0400 (Mon, 06 Jul 2015) $
 *
 * @details
 *            This is the the primary source file for Trigger Routing Unit Manager
 *
 */

/** @addtogroup TRU_Manager Trigger Routing Unit (TRU) Manager
 *  @{

<b>Trigger Routing Unit (TRU) Manager</b>

The TRU provides system-level sequence control without core intervention. The TRU
maps trigger masters (generators of triggers) to trigger slaves (receivers of triggers).
Slave endpoints can be configured to respond to triggers in various ways.

Common applications enabled by the TRU include:

    - Automatically triggering the start of a DMA sequence after a
      sequence from another DMA channel completes
    - Software triggering
    - Synchronization of concurrent activities

The TRU Manager supports the following features

    - Trigger routing of any trigger master to any trigger slave.
    - Software generation of any trigger master ID.

Trigger configuration can be set on a per-buffer basis for all Peripheral DMA data transfer modes
and One-Shot/Circular Memory DMA transfer modes. For DMA driven peripherals, application should
use trigger configuration APIs provided by the corresponding peripheral driver.

Sequence of function calls to route trigger master and slave

    - adi_tru_Init() initializes the TRU service. 
    - adi_tru_ConfigureSlave() to route a trigger master to a trigger slave.
    - Trigger configuration APIs specific to master/slave device(s).
    - Configure master/slave device(s) to act on its trigger settings.
    - adi_tru_RaiseTriggerMaster() or adi_tru_RaiseTriggerMultiple() to generate up to
      four outgoing software-driven triggers per function call.
    - adi_tru_ConfigureSlave() with master id as zero to clear the trigger routing that
      was established earlier.

 */

/*=============  I N C L U D E S   =============*/
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be longer than 31 characters")
#endif

/* TRU Manager includes */
#include <services/tru/adi_tru.h>
/* Include header file with data structure definitions used in TRU manager implementation */
#include "adi_tru_def.h"

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

/*=============  L O C A L    F U N C T I O N    P R O T O T Y P E S =============*/

/*=============  C O D E  =============*/

/**
 * @brief Initializes Trigger Routing Unit, optionally performing a soft reset.
 *
 * Parameters:
 *
 * @param [in]  bReset     'true' to soft-reset the TRU, 'false' to do nothing.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully configured TRU.
 *
 */
ADI_TRU_RESULT  adi_tru_Init(bool bReset)
{
    /* The soft-reset of the TRU is optional */
    if (bReset)
    {
#if defined(ADI_ADSP_CM41X_M4)
        /* Soft-Reset TRU */
        *pREG_TRU1_GCTL |= BITM_TRU_GCTL_RESET;
#else
        /* Soft-Reset TRU */
        *pREG_TRU0_GCTL |= BITM_TRU_GCTL_RESET;
#endif /* ADI_ADSP_CM41X_M4 */
    }

#if defined(ADI_ADSP_CM41X_M4)
    /* Enable TRU */
    *pREG_TRU1_GCTL |= BITM_TRU_GCTL_EN;
#else
    /* Enable TRU */
    *pREG_TRU0_GCTL |= BITM_TRU_GCTL_EN;
#endif /* ADI_ADSP_CM41X_M4 */

    return ADI_TRU_SUCCESS;
}

/**
 * @brief Shuts down the Trigger Routing Unit.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully closed trigger routing unit.
 *
 */
ADI_TRU_RESULT  adi_tru_UnInit(void)
{
#if defined(ADI_ADSP_CM41X_M4)
    /* Disable TRU */
    *pREG_TRU1_GCTL &= ~BITM_TRU_GCTL_EN;
#else
    /* Disable TRU */
    *pREG_TRU0_GCTL &= ~BITM_TRU_GCTL_EN;
#endif /* ADI_ADSP_CM41X_M4 */

    return ADI_TRU_SUCCESS;
}


/**
 * @brief Routes/Maps a single trigger master to a single slave.
 *
 * @details This API will make the given Slave to wait (listen) for the trigger from the
 *          given master. Multiple slaves can wait for the trigger from the single master.
 *          To achieve multiple slave routings to a single master, this API may be called
 *          multiple times with different Slave IDs, but with same Master ID.
 *
 * @param [in]  Slave        Trigger Slave ID.
 * @param [in]  Master       Trigger Master ID.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully mapped a trigger master to slave.
 *
 * @note
 *      The Trigger Master and Slave IDs are defined in processor definition file (defxxxx.h) \n
 *      defBF60x.h - Definition file for ADSP-BF60x Processor family \n
 *      defBF70x.h - Definition file for ADSP-BF70x Processor family \n
 *      defCM40z.h - Definition file for ADSP-CM40z Processor family \n
 *      defCM41x.h - Definition file for ADSP-CM41x Processor family \n
 *      defSC589.h - Definition file for ADSP-SC58x Processor family \n
 *
 */
ADI_TRU_RESULT  adi_tru_ConfigureSlave(
    int32_t            Slave,
    int32_t            Master
)
{
#if defined(ADI_ADSP_CM41X_M4)
    volatile int32_t   (*TriggerSlaveReg)[] = (volatile int32_t(*)[]) REG_TRU1_SSR0;
#else
    volatile int32_t   (*TriggerSlaveReg)[] = (volatile int32_t(*)[]) REG_TRU0_SSR0;
#endif /* ADI_ADSP_CM41X_M4 */

    /* Map trigger master to slave */
    (*TriggerSlaveReg)[Slave] = Master;

    return ADI_TRU_SUCCESS;
}

/**
 * @brief Generate a Master Trigger.
 *
 * @details This API writes the Master Trigger Register (MTR). Any master trigger
 *          (including the software based triggers) can be generated by calling this API.
 *          The trigger will route to the currently routed trigger slave(s),
 *          as configured by #adi_tru_ConfigureSlave().
 *
 * @param [in]  Master    Master Trigger Register (MTR) value to write.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully raised the triggers.
 *
 */
ADI_TRU_RESULT adi_tru_RaiseTriggerMaster(
    int32_t            Master
)
{
#if defined(ADI_ADSP_CM41X_M4)
    /* Configure the Trigger Master register */
    *pREG_TRU1_MTR = (uint32_t)Master;
#else
    /* Configure the Trigger Master register */
    *pREG_TRU0_MTR = (uint32_t)Master;
#endif /* ADI_ADSP_CM41X_M4 */

    return ADI_TRU_SUCCESS;
}

/**
 * @brief Generate up to four Master Triggers.
 *
 * @details This API writes the Master Trigger Register (MTR). Any master trigger
 *          (including the software based triggers) can be generated by calling this API.
 *          Up to four masters may be raised simultaneously, unused master arguments must
 *          set to zero.
 *          The triggers will route to the currently routed trigger slave(s),
 *          as configured by #adi_tru_ConfigureSlave().
 *
 * @param [in]  Master0    Master Trigger to raise.
 * @param [in]  Master1    Master Trigger to raise.
 * @param [in]  Master2    Master Trigger to raise.
 * @param [in]  Master3    Master Trigger to raise.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully raised the triggers.
 *
 */
ADI_TRU_RESULT adi_tru_RaiseTriggerMultiple(
    int32_t            Master0,
    int32_t            Master1,
    int32_t            Master2,
    int32_t            Master3
)
{
    const uint32_t compTriggerVal =
	(((uint32_t)Master0 << (uint32_t)BITP_TRU_MTR_MTR0) & BITM_TRU_MTR_MTR0) |
	(((uint32_t)Master1 << (uint32_t)BITP_TRU_MTR_MTR1) & BITM_TRU_MTR_MTR1) |
	(((uint32_t)Master2 << (uint32_t)BITP_TRU_MTR_MTR2) & BITM_TRU_MTR_MTR2) |
	(((uint32_t)Master3 << (uint32_t)BITP_TRU_MTR_MTR3) & BITM_TRU_MTR_MTR3);

#if defined(ADI_ADSP_CM41X_M4)
    /* Configure the Trigger Master register */
    *pREG_TRU1_MTR = compTriggerVal;
#else
    /* Configure the Trigger Master register */
    *pREG_TRU0_MTR = compTriggerVal;
#endif /* ADI_ADSP_CM41X_M4 */

    return ADI_TRU_SUCCESS;
}

