/*
 * Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.
 * This software is proprietary and confidential.  By using this software you agree
 * to the terms of the associated Analog Devices License Agreement.
 */

/*
 * Use the power service to change processor mode and frequency settings.
 */

#include <services/pwr/adi_pwr.h>

/* ADI initialization header */
#include "PWR_Freq_Mode.h"
extern void SetBaud(void);

static bool bError = false;

static void CheckResult(ADI_PWR_RESULT result, char *message);
static void setDefaultPower(void);

/*
 * Report error if result code is not success
 */
static void CheckResult(ADI_PWR_RESULT result, char *message)
{
#if !defined (__DEBUG_UART__)
    if (result != ADI_PWR_SUCCESS) {
        //DEBUG_PRINT("%s failed\n", message);
        bError = true;
    }
#endif
}

/*
 * Reset processor to default power settings
 */
static void setDefaultPower(void)
{
    adi_pwr_SetClkControlRegister(ADI_PWR_CLK_CTL_DF, DF_DEFAULT);
    adi_pwr_SetClkControlRegister(ADI_PWR_CLK_CTL_MSEL, MSEL_DEFAULT);
    adi_pwr_SetControlRegister(ADI_PWR_CTL_PLL_DISABLE, 0u);
    adi_pwr_SetControlRegister(ADI_PWR_CTL_EXIT_ACTIVE, 1u);
    adi_pwr_SetClkDivideRegister(ADI_PWR_CLK_DIV_SYSSEL, SSEL_DEFAULT);
    adi_pwr_SetClkDivideRegister(ADI_PWR_CLK_DIV_CSEL, CSEL_DEFAULT);
}

/*
 * Power event Callback function
 * Suppress these as someone may want to use in the future
 */
#pragma diag_suppress=Pa050, Pe767, Pe550
static void PWRCallback(void *pCBParam, uint32_t Event, void *pArg)
{
    uint32_t sclk;
    ADI_PWR_MODE  mode;

    switch ( (ADI_PWR_EVENT)Event )
    {
    case ADI_PWR_EVENT_FREQ_PRE_CHANGE:
        sclk = *(uint32_t*)(pArg);
        break;
    case ADI_PWR_EVENT_FREQ_POST_CHANGE:
        sclk = *(uint32_t*)(pArg);
        break;
    case ADI_PWR_EVENT_MODE_PRE_CHANGE:
        mode = (ADI_PWR_MODE)pArg;
        break;
    case ADI_PWR_EVENT_MODE_POST_CHANGE:
        mode = (ADI_PWR_MODE)pArg;
        break;
    default:
        break;
    }
}

/*
 * power_init
 */
uint32_t power_init(  )
{
    uint32_t   fcclk = 0u;
    uint32_t   fsysclk = 0u;
    uint32_t   fsclk0 = 0u;
    uint32_t   fsclk1 = 0u;
    uint32_t   coreclk;
    uint32_t   systemclk;

    ADI_PWR_RESULT result;
    ADI_PWR_MODE  mode;
    int expectedEvents = 0;

    bError = false;

    result = adi_pwr_Init(CLKIN, CORE_MAX, SYSCLK_MAX, VCO_MIN);
    CheckResult(result, "adi_pwr_Init");

    result = adi_pwr_InstallCallback(PWRCallback);
    CheckResult(result, "adi_pwr_InstallCallback");

    /* Set CGU clock select register to SYSCLK/2 */
    result = adi_pwr_SetClkOutSelectRegister(ADI_PWR_CLK_SELECT_CLKOUT, 1); // 1?
    CheckResult(result, "adi_pwr_SetClkOutSelectRegister");

    /* Reset processor to default power settings */
    setDefaultPower();

    /* get the power mode */
    result = adi_pwr_GetPowerMode(&mode);
    CheckResult(result, "adi_pwr_GetPowerMode");
    if (mode != ADI_PWR_MODE_FULL_ON)
    {
        /* set power mode to full on */
        result = adi_pwr_SetPowerMode(ADI_PWR_MODE_FULL_ON);
        CheckResult(result, "adi_pwr_SetPowerMode");

        /* get the power mode */
        result = adi_pwr_GetPowerMode(&mode);
        CheckResult(result, "adi_pwr_GetPowerMode");
        if (mode != ADI_PWR_MODE_FULL_ON)
        {
            bError = true;
        }
    }

    /* set power mode to active */
    result = adi_pwr_SetPowerMode(ADI_PWR_MODE_ACTIVE);
    CheckResult(result, "adi_pwr_SetPowerMode");

    /* get the power mode */
    result = adi_pwr_GetPowerMode(&mode);
    CheckResult(result, "adi_pwr_GetPowerMode");
    if (mode != ADI_PWR_MODE_ACTIVE)
    {
        bError = true;
    }

    /* in active mode the core and system freq are equal to clkin */
    result = adi_pwr_GetCoreFreq(&fcclk);
    result = adi_pwr_GetSystemFreq(&fsysclk,  &fsclk0, &fsclk1);
    if (((int)fcclk != (int)CLKIN) || ((int)fsysclk != (int)CLKIN))
    {
        bError = true;
    }

    /* set power mode to full on */
    result = adi_pwr_SetPowerMode(ADI_PWR_MODE_FULL_ON);
    CheckResult(result, "adi_pwr_SetPowerMode");

    /* set max freq */
    result = adi_pwr_SetFreq(CORE_MAX, SYSCLK_MAX);
    CheckResult(result, "adi_pwr_SetFreq");
    expectedEvents += 2; /* pre and post event */

    result = adi_pwr_GetCoreFreq(&fcclk);
    CheckResult(result, "adi_pwr_GetCoreFreq");

    result = adi_pwr_GetSystemFreq(&fsysclk, &fsclk0, &fsclk1);
    CheckResult(result, "adi_pwr_GetSystemFreq");
    if ((fcclk < (CORE_MAX-CLKIN)) || (fsysclk < (SYSCLK_MAX-CLKIN)))
    {
        bError = true;
    }

    coreclk = CORE_MAX;
    systemclk = 60000000u;

    result = adi_pwr_SetFreq(coreclk, systemclk);
    CheckResult(result, "adi_pwr_SetFreq");

    result = adi_pwr_GetCoreFreq(&fcclk);
    result = adi_pwr_SetClkDivideRegister(ADI_PWR_CLK_DIV_S0SEL, 2);
    result = adi_pwr_SetClkDivideRegister(ADI_PWR_CLK_DIV_S1SEL, 2);
    result = adi_pwr_SetClkDivideRegister(ADI_PWR_CLK_DIV_DSEL, 4);

    /* get the power mode */
    result = adi_pwr_GetPowerMode(&mode);
    CheckResult(result, "adi_pwr_GetPowerMode");
    if (mode != ADI_PWR_MODE_FULL_ON)
    {
        bError = true;
    }

    return bError ? 0 : 1;
}

/*
 * Update the CCLK and SCLK for SRAM test
 */
uint32_t ChangeFreq( uint32_t  coreclk, uint32_t  systemclk)
{
    ADI_PWR_RESULT result;

    result = adi_pwr_SetFreq(coreclk, systemclk);
    CheckResult(result, "adi_pwr_SetFreq");

    return bError ? 0 : 1;
}
